import pytest
from torcp import tortitle
from torcp.torcategory import TorCategory


@pytest.mark.parametrize("test_input, e1, e2, e3, e4", [
    ('权力的游戏.第S01-S08.Game.Of.Thrones.S01-S08.1080p.Blu-Ray.AC3.x265.10bit-Yumi',
     'Game Of Thrones', '', 'S01-S08', ''),
    ('辅佐官：改变世界的人们S01-S02.Chief.of.Staff.2019.1080p.WEB-DL.x265.AC3￡cXcY@FRDS',
     'Chief of Staff', '2019', 'S01-S02', ''),
    ('半暖时光.The.Memory.About.You.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB',
     'The Memory About You', '2021', 'S01', ''),
    ('不惑之旅.To.the.Oak.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB', 'To the Oak',
     '2021', 'S01', ''),
    ('不惑之旅.To.the.Oak.S01.2021.V2.2160p.WEB-DL.AAC.H265-HDSWEB', 'To the Oak',
     '2021', 'S01', ''),
    ('六十一号的恐怖 ：The.Scary.of.Sixty-First.2021.1080p.Blu-ray.AVC.DTS-HD.MA.5.1-GeekArt@CHDBits', 'The Scary of Sixty First', '2021', '', ''),
    ('当家主母.Marvelous.Women.S01.2021.V3.2160p.WEB-DL.AAC.H265-HDSWEB',
     'Marvelous Women', '2021', 'S01', ''),
    ('彩虹宝宝第三季.Rainbow.Ruby.S03.2020.WEB-DL.4k.H265.AAC-HDSWEB', 'Rainbow Ruby', '2020', 'S03', ''),
    ('民警老林的幸福生活.The.Happy.Life.of.People\'s.Policeman.Lao.Lin.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB',
     'The Happy Life of People\'s Policeman Lao Lin', '2021', 'S01', ''),
    ('千古风流人物.Qian.Gu.Feng.Liu.Ren.Wu.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB',
     'Qian Gu Feng Liu Ren Wu', '2021', 'S01', ''),
    ('太平间闹鬼事件 The Haunting in Connecticut 2009 Blu-ray 1080p AVC DTS-HD MA 7.1-Pete@HDSky',
     'The Haunting in Connecticut', '2009', '', ''),
    ('一片冰心在玉壶.Heart.of.Loyalty.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB',
     'Heart of Loyalty', '2021', 'S01', ''),
    ('住在我隔壁的甲方.Party.A.Who.Lives.Beside.Me.S01.2021.1080p.WEB-DL.AAC.H264-HDSWEB',
     'Party A Who Lives Beside Me', '2021', 'S01', ''),
    ('铸匠.The.Builders.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB', 'The Builders',
     '2021', 'S01', ''),
    ('Dinotrux S03E02 1080p Netflix WEB-DL DD 5.1 H.264-AJP69.mkv', 'Dinotrux', '', 'S03', 'E02'),
    ('排球女将.Moero.Attack.1979.Complete.WEB-DL.1080p.H264.DDP.MP3.Mandarin&Japanese-OPS', 'Moero Attack', '1979', '', ''),
    ('CCTV1.Yuan.Meng.Zhong.Guo.De.Yao.Zhong.Hua.20211129.HDTV.1080i.H264-HDSTV.ts',
     'Yuan Meng Zhong Guo De Yao Zhong Hua 20211129', '', '', ''),
    ('CCTV5+.2021.Snooker.UK.Championship.20211129.HDTV.1080i.H264-HDSTV.ts',
     'Snooker UK Championship 20211129', '2021', '', ''),
    ('CCTV5+.2021.World.Table.Tennis.Championship.20211130.HDTV.1080i.H264-HDSTV.ts',
     'World Table Tennis Championship 20211130', '2021', '', ''),
    ('CCTV5.Total.Soccer.20211129.HDTV.1080i.H264-HDSTV.ts',
     'Total Soccer 20211129', '', '', ''),
    ('CCTV9.The.Legend.Of.Film.Ep01-Ep06.HDTV.1080i.H264-HDSTV',
     'The Legend Of Film', '', 'S01', 'Ep01-Ep06'),
    ('The.Boys.S02.2020.1080p.BluRay.DTS.x265-10bit-HDS', 'The Boys', '2020',
     'S02', ''),
    ('X档案.第一季.1993.中英字幕￡CMCT梦幻', 'X档案', '1993', 'S01', ''),
    ('射雕英雄传1983.双语中字(LITV)', '射雕英雄传', '1983', '', ''),
    ('Taxi.4.2007.Director\'s.Cut.2007.Bluray.1080p.x264.DD5.1-wwhhyy@Pter.mkv', 'Taxi 4', '2007', '', ''),
    ('CCTV8HD My Own Swordsman E01-E80 1396x866p H10bits HDTV H264-sammy', 'My Own Swordsman', '', 'S01', 'E01-E80'),
    ('Californication.S01-07.1080p.BluRay.DD5.1.x264-NTb', 'Californication', '', 'S01-07', ''),
    ('Fan.Ren.Xiu.Xian.Zhuan.E32.1080p.WEB-DL.H264.AAC-CHDWEB.mp4', 'Fan Ren Xiu Xian Zhuan', '', 'S01', 'E32'),
    ('Indiana Jones The Complete Adventures 1981-2008 UHD Blu-Ray 2160p&1080p HEVC&AVC Atmos TrueHD 7 1&DTS-HD MA 5 1-CHDBits', 'Indiana Jones The Complete Adventures', '1981-2008', '', ''),
    ('Doctor.X.Surgeon.Michiko.Daimon.S06.1080p.BluRay.x265.10bit.FLAC.MNHD-FRDS',
     'Doctor X Surgeon Michiko Daimon', '', 'S06', ''),
    ('逆世界.Upside.Down.2012.BluRay.1080p.x265.10bit.2Audio.MNHD-FRDS',
     'Upside Down', '2012', '', ''),
    ('野战排.Platoon.1986.BluRay.1080p.x265.10bit.2Audio.MNHD-FRDS', 'Platoon',
     '1986', '', ''),
    ('Stargate.Atlantis.S04.Multi.1080p.BluRay.DTS-HDMA.5.1.H.264-CELESTIS',
     'Stargate Atlantis', '', 'S04', ''),
    ('谍影重重1-5.The.Bourne.2002-2016.1080p.Blu-ray.x265.DTS￡cXcY@FRDS',
     'The Bourne', '2002-2016', '', ''),
    ('豹.1963.JPN.1080p.意大利语中字￡CMCT风潇潇', '豹', '1963', '', ''),
    # ('[迪士尼]2009.飞屋环游记.天外奇迹.冲天救兵.冲天奇兵.3D版.国粤英.原盘中字.BD.Iso', '飞屋环游记', '2009', ''),
    ('过界男女.2013.国粤双语.简繁中字￡CMCT紫卿醺', '过界男女', '2013', '', ''),
    ('Maleficent Mistress of Evil V2 2019 ULTRAHD Blu-ray 2160p HEVC Atmos TrueHD 7.1-sGnb@CHDBits', 'Maleficent Mistress of Evil V2', '2019', '', ''),
    ('金刚狼3殊死一战.Logan.2017.BluRay.1080p.x265.10bit.MNHD-FRDS', 'Logan', '2017',
     '', ''),
    ('托马斯和他的朋友们第18季 第4集_3840x2160_H265_10.51_285.91MB.mp4', '托马斯和他的朋友们', '', 'S18', 'E4'),
    ('(BDMV)Anneke Gronloh - De Regenboog Serie (2009) FLAC-CD] {NL,Telstar B.V,TCD 70316-2}',
     'Anneke Gronloh De Regenboog Serie', '2009', '', ''),
    ('Foundation.2021.S01.2160p.ATVP.WEB-DL.DDP5.1.DV.HEVC-CasStudio',
     'Foundation', '2021', 'S01', ''),
    ('War for the Planet of the Apes 2017 ULTRAHD BluRay 2160p HEVC Atmos TrueHD 7.1-sGnb@CHDBits', 'War for the Planet of the Apes', '2017', '', ''),
    ('我是你的眼.I\'m.Your.Eyes.2016.S01.2160p.WEB-DL.H265.AAC-LeagueWEB',
     'I\'m Your Eyes', '2016', 'S01', ''),
    ('The Fixies Ⅲ 2017 EP01-EP52 1080P WEB-DL AAC x264-GTQing', 'The Fixies Ⅲ', '2017', 'S01', 'EP01-EP52'),
    ('人类星球 Human Planet(1080P)', 'Human Planet', '', '', ''),
    ('The.Owl.House.S01.1080p.DSNP.WEBRip.AAC2.0.x264-LAZY[rartv]', 'The Owl House', '', 'S01', ''),
    ('The.Owl.House.S02E02.Escaping.Expulsion.1080p.HULU.WEBRip.AAC2.0.H264-LAZY[rarbg]', 'The Owl House', '', 'S02', 'E02'),
    ('Guard.Jie.Fang.Xi.2022.S03.1080p.WEB-DL.H264.AAC-TJUPT', 'Guard Jie Fang Xi', '2022', 'S03', ''),
    ('[不能只有我看到的-便利店追女神食谱].Fast&Delicious.2021.1080i.HDTV.H264.DD-PTerTV', 'Fast&Delicious', '2021', '', ''),
    ('Ms.45.1981.720p.BluRay.FLAC1.0.x264-PTer', 'Ms 45', '1981', '', ''),
    ('BTV.The.Forbidden.City.Ep11-Ep12.HDTV.1080i.H264-HDSTV', 'The Forbidden City', '', 'S01', 'Ep11-Ep12'),
    ('HunanTV.Da.Wan.Zai.De.Ye.20211201.HDTV.1080i.H264-HDSTV.ts', 'Da Wan Zai De Ye 20211201', '', '', ''),
    ('2021.FIVB.VNL.CHN.vs.BRA.20210608.1080p.REPACK.WEB-DL.x264.AAC-TJUPT.mp4', 'FIVB VNL CHN vs BRA 20210608', '2021', '', ''),
    ('失落的秘符.第1季', '失落的秘符', '', 'S1', ''),
    ('最后的决斗.The.Last.Duel.2021.1080p.Blu-ray.x265.DTS￡cXcY@FRDS', 'The Last Duel', '2021', '', ''),
    ('现代爱情S02.Modern.Love.2021.1080p.WEB-DL.x265.AC3￡cXcY@FRDS', 'Modern Love', '2021', 'S02', ''),
    # ('สิ่งเล็กเล็กที่เรียกว่า...รัก.A.Little.Thing.Called.Love.AKA.First.Love.2010.WEB-DL.1080p.x264.AAC-PTHome.mp4', 'A Little Thing Called Love', '2010', '', ''),
    ('Top138.英雄本色(4K修复版).A.Better.Tomorrow.1986.REMASTERED.Bluray.1080p.x265.AAC(5.1).2Audios.GREENOTEA', 'A Better Tomorrow', '1986', '', ''),
    ('Weathering.With.You.2019.1080p.NLD.AVC.DTS-HD.MA.5.1-NeoVision', 'Weathering With You', '2019', '', ''),
    ('[BDMV][Bokutachi no Remake][Vol.01-02]', 'Bokutachi no Remake', '', '', ''),
    ('[吸血鬼同盟][Dance In The Vampire Bund][ダンスインザヴァンパイアバンド][BDMV][1080p][DISC×2][GER]', 'Dance In The Vampire Bund', '', '', ''),
    ('1917 2019 V2 ULTRAHD BluRay 2160p HEVC Atmos TrueHD7.1-sGnb@CHDBits', '1917', '2019', '', ''),
    ('[和楽器バンド (Wagakki Band) – TOKYO SINGING [初回限定映像盤 2Blu-ray]][BDMV][1080P][MPEG-4 AVC / LPCM]', '和楽器バンド Wagakki Band TOKYO SINGING', '', '', ''),
    ('[酷爱电影的庞波小姐][Eiga Daisuki Pompo-san][映画大好きポンポさん][BDRip][1920x1040][Movie][x264 Hi10P TrueHD MKV][TTGA]', 'Eiga Daisuki Pompo san', '', '', ''),
    ('[柳林风声][The Wind in the Willows][BDMV][1080p][MOVIE][AVC LPCM][UK]', 'The Wind in the Willows', '', '', ''),
    ('[BanG Dream! Episode of Roselia][劇場版 BanG Dream! Episode of Roselia][BDRip][1920x1080][Movie 01-02 Fin+SP][H264 FLAC DTS-HDMA MKV][自壓]', 'BanG Dream! Episode of Roselia', '', '', ''),
    ('[持续狩猎史莱姆三百年, 不知不觉就练到LV MAX][Slime Taoshite 300-nen, Shiranai Uchi ni Level Max ni Nattemashita][スライム倒して300年、知らないうちにレベルMAXになってました][外挂结构][日英简]', 'Slime Taoshite 300 nen, Shiranai Uchi ni Level Max ni Nattemashita', '', '', ''),
    ('[贾希大人不气馁][The Great Jahy Will Not Be Defeated! / Jahy-sama wa Kujikenai!][ジャヒー様はくじけない!][BDMV][1080p][Vol.1]', 'The Great Jahy Will Not Be Defeated!', '', '', ''),
    ('[银河英雄传说 Die Neue These (2019)][Ginga Eiyuu Densetsu: Die Neue These (2019) / Legend of the Galactic Heroes: Die Neue These (2019)][銀河英雄伝説 Die Neue These (2019)][BDMV][1080p][Vol.4-6 BDx3 + DVDx3 Fin][JP]', 'Ginga Eiyuu Densetsu: Die Neue These 2019', '2019', '', ''),
    ('[堀與宮村][Horimiya][ホリミヤ][加流重灌 (Modded Blu-rays)][1080P][Vol.1-Vol.7 Fin][SweetDreamDay]', 'Horimiya', '', '', ''),
    ('[剧场版 Healin\' Good 光之美少女 梦想的小镇心动不已! GoGo! 大变身!!][Eiga Healin\' Good Precure Yume no Machi de Kyun! Tto Go Go! Dai Henshin!!][映画ヒーリングっど プリキュア ゆめのまちでキュン!っとGoGo!大変身!!/(短編)映画トロピカル~ジュ!プリキュアとびこめ!コラボダンスパーティ!][BDMV][MOVIE][U2娘@Share]', 'Eiga Healin\' Good Precure Yume no Machi de Kyun! Tto Go Go! Dai Henshin!!', '', '', ''),
    ('[鲁邦三世VS名侦探柯南 THE MOVIE]Lupin the 3rd vs Detective Conan THE MOVIE / Lupin Sansei vs. Meitantei Conan The Movie[ルパン三世vs名探偵コナン THE MOVIE][BDMV][1080p][MOVIE][AVC][ITA]', 'Lupin the 3rd vs Detective Conan THE MOVIE', '', '', ''),
    ('Metal Gear Solid 3D - Snake Eater (USA) (En,Fr,Es).zip', 'Metal Gear Solid 3D Snake Eater USA En Fr Es', '', '', ''),
    ('[Moozzi2] Assassins Pride [ x265-10Bit Ver. ] - TV + SP', 'Assassins Pride', '', '', ''),
    ('CC_(1960-1964)_The Kennedy Films of Robert Drew & Associates', 'The Kennedy Films of Robert Drew & Associates', '1960-1964', '', ''),
    ('[TV][NPMS][Cocomelon 第三季][CoComelon.S03.1080p.NF.WEB-DL.DDP2.0.H.264-NPMS][MKV]', 'CoComelon', '', 'S03', ''),
    ('[WARP276] Danny Brown - Atrocity Exhibition (2016) [FLAC]', 'Danny Brown Atrocity Exhibition 2016', '2016', '', ''),
    ('The Beatles - Rubber Soul (Remastered) 1965 - FLAC 16bit 44.1khz', 'The Beatles Rubber Soul', '1965', '', ''),
    ('Marina - Ancient Dreams In A Modern Land (2021) - WEB FLAC', 'Marina Ancient Dreams In A Modern Land', '2021', '', ''),
    ('郁可唯 - 蓝短裤 2010 FLAC', '郁可唯 蓝短裤', '2010', '', ''),
    ('[皇后高品质音乐]陈慧琳-微光[分轨FLAC]', '陈慧琳 微光', '', '', ''),
    ('Stefanie Sun (孫燕姿) - Against the Light (逆光) (2007) - FLAC', 'Stefanie Sun 孫燕姿 Against the Light 逆光', '2007', '', ''),
    ('Pink Floyd - The Final Cut (1983) [FLAC] {72435-76734-2-6}', 'Pink Floyd The Final Cut', '1983', '', ''),
    ('[2021.05.28] にじさんじ - PALETTE 003 - Virtual Strike (フル・シングル) [FLAC]', 'にじさんじ PALETTE 003 Virtual Strike フル・シングル', '2021', '', ''),
    ('Peggy_Lee-The_Legendary_Peggy_Lee-3CD-FLAC-1999-FLACME', 'Peggy Lee The Legendary Peggy Lee', '1999', '', ''),
    ('Bob Dylan - Bringing It All Back Home (1965) [FLAC] {2013 MFSL}', 'Bob Dylan Bringing It All Back Home', '1965', '', ''),
    ('张国荣 - H₂O 1984 480i MPEG AC3-PTerMV.VOB', '张国荣 H₂O', '1984', '', ''),
    ('WINNER - EVERYD4Y (FLAC)', 'WINNER EVERYD4Y', '', '', ''),
    ('Dee-Mack-Doin_It_My_Way-CD-FLAC-1995-AUDiOFiLE', 'Dee Mack Doin It My Way', '1995', '', ''),
    ('Andrea Bocelli-Super Hits-2CD-WEB-2004-playMUSIC', 'Andrea Bocelli Super Hits', '2004', '', ''),
    ('范玮琪 - 我们的纪念日 FLAC', '范玮琪 我们的纪念日', '', '', ''),
    ('[EAC][180613][SINGLE][藍井エイル][流星／約束][VVCL-1252~4][FLAC+CUE+LOG+PNG+ISO]', '藍井エイル', '', '', ''), 
    ('[EAC][170927][SINGLE][V.A.][SWORD ART ONLINE THE MOVIE ORDINAL SCALE CHARACTER SONGS][ANZX-14002][FLAC+CUE+LOG]', 'SWORD ART ONLINE THE MOVIE ORDINAL SCALE CHARACTER SONGS', '', '', ''),
    ('[EAC][170927][ALBUM][能登麻美子][地獄少女 ENDING SONG COLLECTION][SBCV-80273][FLAC+CUE+LOG+PNG]', '地獄少女 ENDING SONG COLLECTION', '', '', ''),
    ('[银魂°(第3期)][Gintama Season3][銀魂°][BDrip][1920x1080][TV 01-51(266-316)Fin+SP][H264 FLAC MKV][自壓(付相關專輯)]', 'Gintama Season3', '', '', ''),
    ('[银魂./银魂 第四期/银魂 烙阳决战篇][Gintama./Gintama Season 4][銀魂./銀魂. 烙陽決戦篇][BDMV][Vol.03][JPN][MAL]', 'Gintama.', '', '', ''),
    ('[銀魂°][Gintama][銀魂゜][BDMV][1080p][Season 3 Parts 1+2 Fin][USA][自抓]', 'Gintama', '', '', ''),
    ('Megazone 23 1985 BD', 'Megazone 23', '1985', '', ''), 
    ('BD-50_A_PORTRAIT_OF_SHUNKIN_1976_BC', 'A PORTRAIT OF SHUNKIN', '1976', '', ''),
    ('Dave (1993) BD25', 'Dave', '1993', '', ''),
    ('SHES_FUNNY_THAT_WAY', 'SHES FUNNY THAT WAY', '', '', ''),
    ('Sabotage [BD25] (1939)', 'Sabotage', '1939', '', ''),
    ('A Raisin in the Sun (1961) NTSC DVD9+DVD5', 'A Raisin in the Sun', '1961', '', ''),
    ('Little Nikita [1988] by Richard Benjamin [R2]', 'Little Nikita', '1988', '', ''),
    ('1991.2018.BluRay.Remux.1080p.AVC.DTS-HD.MA.5.1-HDH', '1991', '2018', '', ''),
    ('[备长炭][Binchou-tan][びんちょうタン][DVDISO][720x480][Vol.1-Vol.3 Fin][R2J]', 'Binchou tan', '', '', ''),
    ('[龙珠Z剧场版 01-13][Dragonball Z – The Movies –][ドラゴンボールZ 劇場版][BDMV][1080p][BD-BOX 1-3 Disc*6 Fin][MPEG-4 AVC][GER][iFPD]', 'Dragonball Z The Movies', '', '', ''),
    ('[EAC][合集][菅野洋子作曲专辑合集][Vol.5][17张]', '菅野洋子作曲专辑合集', '', '', ''),
    ('[To LOVEる][To LOVERu][HDTVrip][1280x720][01-26FIN][MKV][澄空学园字幕组内嵌]', 'To LOVERu', '', '', ''),
    ('Ikumi Ogasawara - Tears Of Joy (2019) [DSF] DSD512', 'Ikumi Ogasawara Tears Of Joy', '2019', '', ''),
    ('VA-The_Best_80s_Album_In_The_World_Ever-3CD-FLAC-2020-ERP', 'VA The Best 80s Album In The World Ever', '2020', '', ''),
    ('Chihayafuru Part 3 2018 JPN Blu-ray 1080p AVC DTS-HD MA 5.1-UserExperience-Raws', 'Chihayafuru Part 3', '2018', '', ''),
    ('[Hi-Res] Evan Call - VIOLET EVERGARDEN_Echo Through Eternity [FLAC 96kHz-24bit]', 'Evan Call VIOLET EVERGARDEN Echo Through Eternity', '', '', ''),
    ('10000 Chopin, Liszt, Debussy Piano Peaces (Miyuji Kaneko)', '10000 Chopin Liszt Debussy Piano Peaces Miyuji Kaneko', '', '', ''),
    ('2004-原来我爱你那么多 新歌+精选 2CD[引进版][WAV]', '原来我爱你那么多 新歌 精选', '2004', '', ''),
    ('滅火器 (Fire EX.) - On Fire Day 2020 - Fire Next 新篇章：滅火器20 週年演唱會 (2021)', '滅火器 Fire EX On Fire Day 2020 Fire Next 新篇章：滅火器20 週年演唱會', '2021', '', ''),
    ('(2012.12.21) Kenichiro Nishihara - Iluminus (flac)', 'Kenichiro Nishihara Iluminus', '2012', '', ''),
    ('Twice - #TWICE (CD-FLAC)', 'Twice #TWICE', '', '', ''),
    ('风潮唱片-《马修.连恩系列-驯鹿宣言》(Caribou Commons)专辑[APE整轨]', '风潮唱片 《马修 连恩系列 驯鹿宣言》 Caribou Commons 专辑', '', '', ''),
    ('Bach - Vionlin Concertos - Isabelle Faust {HMM 902335.36} (2019) [FLAC]', 'Bach Vionlin Concertos Isabelle Faust', '2019', '', ''),
    ('[迪士尼]2015.恐龙当家.恐龙大时代.美好的恐龙世界.善良的恐龙.恐龙管家.3D港版.国语简繁中字.BD.ISO', '恐龙当家 恐龙大时代 美好的恐龙世界 善良的恐龙 恐龙管家', '2015', '', ''),
    # ('E3.2018.Press.Conference.WEB-DL.1080p.H264.AAC-CMCTV', 'E3', '2018', 'S01', 'E3'),
    ('踢馆秘笈（第六季）—测试样片', '踢馆秘笈', '', 'S06', ''),
    ('海绵宝宝 国语 12季', '海绵宝宝', '', 'S12', ''),
    ('[阿涅斯论瓦尔达].Varda.by.Agnès.2019.CC.BluRay.1080p.x264.DTS-CMCT', 'Varda by Agnès', '2019', '', ''), 
    ('[艾娃].Ava.2020.AUS.BluRay.1080p.x264.DTS-CMCT', 'Ava', '2020', '', ''),
    ('[Jose与虎与鱼们].josee.the.tiger.and.the.fish.2003.KOR.BluRay.1080p.x264.FLAC-CMCT.mkv', 'josee the tiger and the fish', '2003', '', ''),
    ('No.33_指环王3.魔戒三部曲：国王归来.TLOTR.The.Return.of.the.King.EE.2003.加长版.x265.BD1080P.国英双语.特效中英字幕.mp4', 'TLOTR The Return of the King EE', '2003', '', ''),
    ('No.238_千钧一发(变种异煞) Gattaca (1997).BD720P.国英双语.中英双 字.mp4', 'Gattaca', '1997', '', ''),
    ('胜者即是正义SP.2013.720p.日语.简体中字￡WiKi(feat.CMCT)', '胜者即是正义SP', '2013', '', ''),
    ('七个世界，一个星球.全7集.2019.1080p.国英双语.双国配.中英字幕￡CMCT小鱼', '七个世界，一个星球', '2019', '', ''),
    ('音乐之声.五十周年纪念版.1965.1080p.国英双语.中英字幕￡CMCT风潇潇', '音乐之声', '1965', '', ''),
    ('The Borgias Disc2', 'The Borgias', '', '', ''),
    ('[故宫].全12集.简繁特效.2005.AVC.1080p.HDTV.H264.AC-3.DD2.0-@szdqwx', '故宫', '2005', '', ''),
    ('花仙子 全50集+剧场版.1080p.国日双语.中文字幕', '花仙子', '', '', ''),
    ('守夜号S01.Vigil.E01-E06.2021.1080p.Blu-ray.x265.AC3￡cXcY@FRDS', 'Vigil', '2021', 'S01', ''),
    ('黄石前传S01.1883.2021.1080p.WEB-DL.x265.AC3￡cXcY@FRDS', '1883', '2021', 'S01', ''),
    ('Who\'s.the.Murderer.2022.S07E05A.PartA.1080p.WEB-DL.H264.AAC-TJUPT', 'Who\'s the Murderer', '2022', 'S07', 'E05'),
    ('[二十五，二十一].Twenty.Five.Twenty.One.2022.1080p.S01E03.WEB-DL.H264.AAC-PTerWEB.mp4', 'Twenty Five Twenty One', '2022', 'S01', 'E03'),
    ('威尼斯疑魂.4K修复版.1973.CC.1080p.中英字幕￡CMCT风潇潇', '威尼斯疑魂', '1973', '', ''),
    ('奇异博士2：疯狂多元宇宙.Doctor.Strange.in.the.Multiverse.of.Madness.2022.2160p.UHD.BluRay.2160p.x265.10bit.HDR.mUHD-FRDS', 'Doctor Strange in the Multiverse of Madness', '2022', '', ''),
    # ('[劫与罪].Heist.2021.S01.S01E03.NF.WEB-DL.1080p.H264.DDP.5.1-CMCTV.mp4', 'Heist', '2021', 'S01', 'E03'),
    # ('Grateful Dead - Listen to the River：St. Louis \'71 \'72 \'73 (Live) (2021) (24-192)', 'Grateful Dead - Listen to the River：St. Louis', '2021', '', ''),
    # ('32.迪卡唱片.-.《极致：勃拉姆斯选集》.(5CDs.Box-set)', '32 迪卡唱片 《极致：勃拉姆斯选集》', '', '', ''),
])
def test_parseTorName(test_input, e1, e2, e3, e4):
    tt = tortitle.TorTitle(test_input)
    assert e1 == tt.title and e2 == tt.yearstr and e3 == tt.season and e4 == tt.episode


@pytest.mark.parametrize("test_input, e1, e2", [
    ('权力的游戏.第S01-S08.Game.Of.Thrones.S01-S08.1080p.Blu-Ray.AC3.x265.10bit-Yumi',
     'TV', 'Yumi'),
    ('辅佐官：改变世界的人们S01-S02.Chief.of.Staff.2019.1080p.WEB-DL.x265.AC3￡cXcY@FRDS',
     'TV', 'FRDS'),
    ('半暖时光.The.Memory.About.You.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB', 'TV',
     'HDSWEB'),
    ('排球女将.Moero.Attack.1979.Complete.WEB-DL.1080p.H264.DDP.MP3.Mandarin&Japanese-OPS', 'TV', 'OPS'),
    ('The.Smurfs.2.2013.3Audio.DTS-HD.MA5.1.x265.10bit-BeiTai.mkv', 'MovieEncode', 'BeiTai'),
    ('Death.on.the.Nile.1978.1080p.BluRay.FLAC.2.0.x264-PTer.mkv', 'MovieEncode', 'PTer'),    
    ('不惑之旅.To.the.Oak.S01.2021.2160p.WEB-DL.AAC.H265-HDSWEB', 'TV', 'HDSWEB'),
    ('BDRemux+Hakumei+to+Mikochi+JP+7³ACG@OurBits', 'Other', 'OurBits'),
    ('FIFA.WORLD.CUP.QUALIFIERS.ENG.VS.ALB.20211113.1080i.HDTV.H264.DD-PTerTV.ts', 'HDTV', 'PTerTV'),
    ('Babymother.1998.1080p.BluRay.REMUX.AVC.FLAC.2.0-BLURANiUM.mkv', 'MovieRemux', 'BLURANiUM'),
    ('春花秋月未了情.Breezy.1973.1080p.BluRay.x265.10bit.FLAC.MNHD-FRDS', 'MovieEncode', 'FRDS'),
    ('Michael Jackson - The Mystery Of HIStory (1997) [FLAC]', 'Music', None),
    ('New_Order-(The_Rest_Of)_New_Order-CD-FLAC-1995-WREMiX', 'Music', 'WREMiX'),
    ('VA-Kill_Bill_Vol_2-(9362-48676-2)-CD-FLAC-2004', 'Music', '2004'),
    ('The Blue Diamonds - Het Beste Van (1988) [FLAC-CD] {WG,RM,Philips,834 484-2}', 'Music', None),
    ('Commodores - Caught In The Act (1975) [FLAC] {24-192 HDTracks}', 'Music', None),
    ('BTV.The.Forbidden.City.Ep11-Ep12.HDTV.1080i.H264-HDSTV', 'TV', 'HDSTV'),
    ('[漫游·张靓颖沉浸式虚拟音乐会·完整版].Roaming·Jane Zhang Immersive Virtual Concert.2021.1080p.WEB-DL.AVC.AAC-QHstudIo', 'MV', 'QHstudIo'),
    ('OOOPS2.2020.BluRay.1080p.2Audio.TrueHD.5.1.x265.10bit-BeiTai', 'MovieEncode', 'BeiTai'),
    ('现代爱情S02.Modern.Love.2021.1080p.WEB-DL.x265.AC3￡cXcY@FRDS', 'TV', 'FRDS'),
    ('The.Green.Planet.S01E01.Tropical.Worlds.2160p.iP.WEB-DL.AAC2.0.H.265-NTb.mkv', 'TV', 'NTb'),
    ('Adele.-.[Live.At.The.Royal.Albert.Hall].2011.Blu-Ray.1080I.H264.DTS-HDMA.-TTG', 'MV', 'TTG'),
    ('Aimer - DAWN (2015) {24bit, WEB} [FLAC]', 'Music', None),
    ('Bad Boys for Life 2020 V2 ULTRAHD BluRay 2160p HEVC DTS-X 7.1-sGnb@CHDBits', 'MovieBDMV4K', 'CHDBits'),
    ('The Fixies Ⅲ 2017 EP01-EP52 1080P WEB-DL AAC x264-GTQing', 'TV', 'GTQing'),
    ('[故宫].全12集.简繁特效.2005.AVC.1080p.HDTV.H264.AC-3.DD2.0-@szdqwx', 'HDTV', 'szdqwx'),
    ('Fiksik 螺丝钉1-3季 1080P 国语中字', 'TV', None),
    ('花仙子全50集+剧场版.1080p.国日双语.中文字幕', 'TV', None),
    ('[猫和老鼠1992电影版].Tom.and.Jerry.The.Movie.1992.WEB-DL.1080p.HEVC.AAC-CMCTV.mp4', 'MovieWebdl', 'CMCTV'),
    ('Lucile Boulanger - Bach & Abel_ Solo [FLAC 192kHz-24bit]', 'Music', None),
    ('Dana Zemtsov & Anna Fedorova - Silhouettes 2020 DSF', 'Music', None),
    ('Sabotage [BD25] (1939)', 'MovieBDMV', None),
    ('QUINTET 1979 DVD9 NTSC', 'MovieDVD', None),
    ('RETURN_TO_NEVER_LAND DVD', 'MovieDVD', None),
    ('My.Blueberry.Nights.2007.NTSC.DVD9', 'MovieDVD', None),
    ('Megazone 23 1985 BD', 'MovieBDMV', None),
    ('Metal Gear Solid 3D - Snake Eater (USA) (En,Fr,Es).zip', 'Archive', None),
    ('To.Sir.With.Love.II.1996.DVD.NTSC-CL8', 'MovieDVD', 'CL8'),
    ('Babka_Yozhka_i_drugiye_2008_BD', 'MovieBDMV', None),
    ('别有用心的单身女.Cunning.Single.Lady.S01E05.1080p.NF.WEB-DL.DDP2.0.x264-Ao.mkv', 'TV', 'Ao'),
    ('The.Ice.Age.Adventures.of.Buck.Wild.2022.2160p.DSNP.WEB-DL.DDP5.1.Atmos.DV.H.265-Lee@CHDWEB.mp4', 'MovieWebdl', 'CHDWEB'),
    ('我爱冒牌机器女友.I.Love.Fake.Robot.Girlfriends.2021.1080p.WEB-DL.H264.AAC-CHDWEB.mp4', 'MovieWebdl', 'CHDWEB'),
    ('譚詠麟 - 情人 1994 480i DVD MPEG AC3-PTerMV.vob', 'MV', 'PTerMV'),
    ('[局内人].Inside.Men.2015.BluRay.1080p.x264.DTS-CMCT.mkv', 'MovieEncode', 'CMCT'),
    ('Megazone 23 1985 BD', 'MovieBDMV', None), 
    ('BD-50_A_PORTRAIT_OF_SHUNKIN_1976_BC', 'MovieBDMV', None),
    ('Dave (1993) BD25', 'MovieBDMV', None),
    ('Sabotage [BD25] (1939)', 'MovieBDMV', None),
    ('The Great Australian Bake Off S02 WEB 720p', 'TV', None),
    ('Moses.Storm.Trash.White.2022.720p.WEB.h264-OPUS.mkv', 'MovieWebdl', 'OPUS'),
    ('Choose Your Own Adventure- Abominable Snowman 2006 DVD9', 'MovieDVD', None),
    ('Tron (DVD9)', 'MovieDVD', None),
    ('A Raisin in the Sun (1961) NTSC DVD9+DVD5', 'MovieDVD', None),
    ('Little Nikita [1988] by Richard Benjamin [R2]', 'Other', None),
    ('Shooting Party, The (1984) (PAL DVDR)', 'MovieDVD', None),
    ('The Hillside Strangler.2004.Dvd9-CG', 'MovieDVD', 'CG'),
    ('2002 - In Violet Light - \'15 Hi-Res @ 24~96 (flac)', 'Music', None),
    ('谭咏麟94纯金曲演唱会.Alan.Tan.live.in.HongKong.1994.D9&LD.HEVC.10bit.2Audio.MiniSD-XPcl@PTer', 'MV', 'PTer'),
    ('人类星球 Human Planet(1080P)', 'Movie', None),
    ('BoA.-.Camo.1080p.Bugs.AVC.AAC-HDHome.mp4', 'Movie', 'HDHome'),
    ('米老鼠2013.Mickey.Mouse.2013.S02E17.HR-HDTV.x264.AAC.Chs.Eng-Deefun迪幻字幕组.mkv', 'TV', 'Deefun迪幻字幕组'),
    # ('[AI-Raws] HELLO WORLD (BD HEVC 3840x2160 yuv444p10le Atmos／FLAC／FLAC[バリアフリー] 日本語字幕)[52F61BDB].mkv', 'MovieBDMV', None),
    # ('Sonata Arctica - Acoustic Adventures  - Volume One (2022)', 'Music', None),
    # ('Sara K.-No Cover-Chesky-0196', 'Music', None),
])
def test_categoryByName(test_input, e1, e2):
    tc = TorCategory(test_input)
    assert tc.ccfcat == e1 and tc.group == e2



# -*- coding: utf-8 -*-
import re
from tmdbv3api import TMDb, Movie, TV, Search, Find
# from tmdbv3api.objs.find import Find

from torcp import tortitle
from torcp.torcategory import TorCategory


def transFromCCFCat(cat):
    if re.match(r'(Movie)', cat, re.I):
        return 'movie'
    elif re.match(r'(TV)', cat, re.I):
        return 'tv'
    else:
        return cat


def transToCCFCat(mediatype, originCat):
    if mediatype == 'tv':
        return 'TV'
    elif mediatype == 'movie':
        if not re.match(r'(movie)', originCat, re.I):
            return 'Movie'
    return originCat


def tryint(instr):
    try:
        string_int = int(instr)
    except ValueError:    
        string_int = 0
    return string_int

def parseTMDbStr(tmdbstr):
    if tmdbstr.isnumeric():
        return '', tmdbstr
    m = re.search(r'(m(ovie)?|t(v)?)[-_]?(\d+)', tmdbstr.strip(), flags=re.A | re.I)
    if m:
        catstr = 'movie' if m[1].startswith('m') else 'tv'
        return catstr, m[4]
    else:
        return '', ''

class TMDbNameParser():
    def __init__(self, tmdb_api_key, tmdb_lang, ccfcat_hard=None):
        self.ccfcatHard = ccfcat_hard
        self.ccfcat = ''
        self.title = ''
        self.year = 0
        self.tmdbid = 0
        self.season = ''
        self.episode = ''
        self.cntitle = ''
        self.resolution = ''
        self.group = ''
        self.tmdbcat = ''
        self.original_language = ''
        self.popularity = 0
        self.poster_path = ''
        self.genre_ids =[]

        if tmdb_api_key:
            self.tmdb = TMDb()
            self.tmdb.api_key = tmdb_api_key
            self.tmdb.language = tmdb_lang
        else:
            self.tmdb = None
            # self.tmdb.api_key = None

    def clearData(self):
        self.ccfcat = ''
        self.title = ''
        self.year = 0
        self.tmdbid = -1
        self.season = ''
        self.episode = ''
        self.cntitle = ''
        self.resolution = ''
        self.group = ''
        self.tmdbcat = ''
        self.original_language = ''
        self.popularity = 0
        self.poster_path = ''
        self.genre_ids =[]

    def parse(self, torname, useTMDb=False, hasIMDbId=None, hasTMDbId=None):
        self.clearData()
        tc = TorCategory(torname)
        self.ccfcat, self.group = tc.ccfcat, tc.group
        self.resolution = tc.resolution
        tt = tortitle.TorTitle(torname)
        self.title, parseYear, self.season, self.episode, self.cntitle = tt.title, tt.yearstr, tt.season, tt.episode, tt.cntitle 
        self.year = tryint(parseYear)

        if self.season and (self.ccfcat != 'TV'):
            # print('Category fixed: ' + movieItem)
            self.ccfcat = 'TV'
        if self.ccfcat == 'TV':
            self.season = self.fixSeasonName(self.season)

        if self.ccfcatHard:
            self.ccfcat = self.ccfcatHard

        self.tmdbcat = transFromCCFCat(self.ccfcat)

        if useTMDb:
            if hasTMDbId:
                cat, tmdbstr = parseTMDbStr(hasTMDbId)
                if tmdbstr:
                    tmdbid, title, year = self.searchTMDbByTMDbId(cat, tmdbstr)
                    if tmdbid > 0:
                        print(tmdbid, title, self.ccfcat, self.year)
                        return True
            if hasIMDbId:
                tmdbid, title, year = self.searchTMDbByIMDbId(hasIMDbId)
                if tmdbid > 0:
                    print(tmdbid, title, self.ccfcat, self.year)
                    return
            if not self.checkNameContainsId(torname):
                if self.tmdbcat in ['tv', 'movie', 'Other', 'HDTV']:
                    self.searchTMDb(self.title, self.tmdbcat,
                                    parseYear, self.cntitle)
            self.ccfcat = transToCCFCat(self.tmdbcat, self.ccfcat)
            
    def fixSeasonName(self, seasonStr):
        if re.match(r'^Ep?\d+(-Ep?\d+)?$', seasonStr,
                    flags=re.I) or not seasonStr:
            return 'S01'
        else:
            return seasonStr.upper()

    # def verifyYear(self, resultDate, checkYear, cat):
    #     match = False
    #     resyear = checkYear
    #     m = re.match(r'^(\d+)\b', resultDate)
    #     if m:
    #         resyear = m.group(0)
    #         intyear = int(resyear)
    #         if cat == 'tv':
    #             match = not (self.season == 'S01' and self.year and self.year not in [str(intyear-1), str(intyear), str(intyear+1)])
    #         else:
    #             match = not self.year or (self.year in [str(intyear-1), str(intyear), str(intyear+1)])
    #     if match:
    #         self.year = resyear
    #     return match

    def saveTmdbTVResultMatch(self, result):
        if result:
            if hasattr(result, 'name'):
                self.title = result.name
                # print('name: ' + result.name)
            elif hasattr(result, 'original_name'):
                self.title = result.original_name
                # print('original_name: ' + result.original_name)
            self.tmdbid = result.id
            self.tmdbcat = 'tv'
            if hasattr(result, 'original_language'):
                if result.original_language == 'zh':
                    self.original_language = 'cn'
                else:
                    self.original_language = result.original_language
            if hasattr(result, 'popularity'):
                self.popularity = result.popularity
            if hasattr(result, 'poster_path'):
                self.poster_path = result.poster_path
            if hasattr(result, 'first_air_date'):
                self.year = self.getYear(result.first_air_date)
            elif hasattr(result, 'release_date'):
                self.year = self.getYear(result.release_date)
            else:
                self.year = 0
            if hasattr(result, 'genre_ids'):
                self.genre_ids = result.genre_ids
            print('Found [%d]: %s' % (self.tmdbid, self.title))
        else:
            print('\033[33mNot match in tmdb: [%s]\033[0m ' % (self.title))

        return result is not None

    def saveTmdbMovieResult(self, result):
        if hasattr(result, 'title'):
            self.title = result.title
        elif hasattr(result, 'original_title'):
            self.title = result.original_title
        # if hasattr(result, 'media_type'):
        #     self.ccfcat = transToCCFCat(result.media_type, self.ccfcat)
        self.tmdbid = result.id
        self.tmdbcat = 'movie'
        if hasattr(result, 'original_language'):
            if result.original_language == 'zh':
                self.original_language = 'cn'
            else:
                self.original_language = result.original_language
        if hasattr(result, 'popularity'):
            self.popularity = result.popularity
        if hasattr(result, 'poster_path'):
            self.poster_path = result.poster_path
        if hasattr(result, 'release_date'):
            self.year = self.getYear(result.release_date)
        elif hasattr(result, 'first_air_date'):
            self.year = self.getYear(result.first_air_date)
        else:
            self.year = 0
        if hasattr(result, 'genre_ids'):
            self.genre_ids = result.genre_ids
        
        print('Found [%d]: %s' % (self.tmdbid, self.title))
        return True

    def saveTmdbMultiResult(self, result):
        if hasattr(result, 'media_type'):
            self.imdbcat = result.media_type
            if result.media_type == 'tv':
                self.saveTmdbTVResultMatch(result)
            elif result.media_type == 'movie':
                self.saveTmdbMovieResult(result)
            else:
                print('Unknow media_type %s ' % result.media_type)
        return

    # def imdbMultiQuery(self, title, year=None):
    #     search = Search()
    #     return search.multi({"query": title, "year": year, "page": 1})

    # def sortByPopularity(resultList):
    #     newlist = sorted(resultList, key=lambda x: x.popularity, reverse=True)

    def getYear(self, datestr):
        intyear = 0
        m2 = re.search(
            r'\b((19\d{2}\b|20\d{2})(-19\d{2}|-20\d{2})?)',
            datestr,
            flags=re.A | re.I)
        if m2:
            yearstr = m2.group(2)
            intyear = tryint(yearstr)
        return intyear

    def getTitle(self, result):
        tt = ''
        if hasattr(result, 'name'):
            tt = result.name
        elif hasattr(result, 'title'):
            tt = result.title
        elif hasattr(result, 'original_name'):
            tt = result.original_name
        elif hasattr(result, 'original_title'):
            tt = result.original_title
        return tt


    def containsCJK(self, str):
        return re.search(r'[\u4e00-\u9fa5]', str)

    def findYearMatch(self, results, year, strict=True):
        matchList = []
        for result in results:
            if year == 0:
                matchList.append(result)
                continue

            datestr = ''
            if hasattr(result, 'first_air_date'):
                datestr = result.first_air_date
            elif hasattr(result, 'release_date'):
                datestr = result.release_date

            resyear = self.getYear(datestr)
                # return result

            if strict:
                if resyear == year:
                    matchList.append(result)
                    continue
            else:
                if resyear in [year-3, year-2, year-1, year, year+1]:
                    self.year = resyear
                    matchList.append(result)
                    continue

        if len(matchList) > 0:
            # prefer item with CJK
            if self.tmdb.language == 'zh-CN':
                for item in matchList[:3]:
                    tt = self.getTitle(item)
                    if not tt:
                        continue
                    if self.containsCJK(tt):
                        return item
            return matchList[0]
        return None

    def selectOrder(self, cntitle, cuttitle, list):
        if len(cntitle) < 3 and len(cuttitle)> 5:
            list[0], list[1] = list[1], list[0]
            return list
        else:
            return list

    def replaceRomanNum(self, titlestr):
        # no I and X
        romanNum = [ (r'\bII\b', '2'), (r'\bIII\b', '3'), (r'\bIV\b', '4'), (r'\bV\b', '5'), (r'\bVI\b', '6'), (r'\bVII\b', '7'), (r'\bVIII\b', '8'),
                    (r'\bIX\b', '9'), (r'\bXI\b', '11'), (r'\bXII\b', '12'), (r'\bXIII\b', '13'), (r'\bXIV\b', '14'), (r'\bXV\b', '15'), (r'\bXVI\b', '16')]
        for s in romanNum:
            titlestr = re.sub(s[0], s[1], titlestr, flags=re.A)
        return titlestr

    def searchTMDb(self, title, cat=None, parseYearStr=None, cntitle=None):
        searchList = []
        if title == cntitle:
            cntitle = ''
        cuttitle = re.sub(r'\b(Extended|Anthology|Trilogy|Quadrilogy|Tetralogy|Collections?)\s*$', '', title, flags=re.I)
        cuttitle = re.sub(r'\b(Extended|HD|S\d+|E\d+|V\d+|4K|DVD|CORRECTED|UnCut|SP)\s*$', '', cuttitle, flags=re.I)
        cuttitle = re.sub(r'^\s*(剧集|BBC：?|TLOTR|Jade|Documentary|【[^】]*】)', '', cuttitle, flags=re.I)
        cuttitle = re.sub(r'(\d+部曲|全\d+集.*|原盘|系列|\s[^\s]*压制.*)\s*$', '', cuttitle, flags=re.I)
        cuttitle = re.sub(r'(\b国粤双语|[\b\(]?\w+版|\b\d+集全).*$', '', cuttitle, flags=re.I)
        cuttitle = re.sub(r'(The[\s\.]*(Complete\w*|Drama\w*|Animate\w*)?[\s\.]*Series|The\s*Movie)\s*$', '', cuttitle, flags=re.I)
        cuttitle = re.sub(r'\b(Season\s?\d+)\b', '', cuttitle, flags=re.I)
        if cntitle:
            cntitle = re.sub(r'(\d+部曲|全\d+集.*|原盘|系列|\s[^\s]*压制.*)\s*$', '', cntitle, flags=re.I)
            cntitle = re.sub(r'(\b国粤双语|[\b\(]?\w+版|\b\d+集全).*$', '', cntitle, flags=re.I)

        cuttitle = self.replaceRomanNum(cuttitle)

        m1 = re.search(r'the movie\s*$', cuttitle, flags=re.A | re.I)        
        if m1 and m1.span(0)[0] > 0:
            cuttitle = cuttitle[:m1.span(0)[0]].strip()
            cat = 'movie'

        m2 = re.search(
            r'\b((19\d{2}\b|20\d{2})(-19\d{2}|-20\d{2})?)\b(?!.*\b\d{4}\b.*)',
            cuttitle,
            flags=re.A | re.I)
        if m2 and m2.span(1)[0] > 0:
            cuttitle = cuttitle[:m2.span(1)[0]].strip()
            cuttitle2 = cuttitle[m2.span(1)[1]:].strip()

        intyear = self.getYear(parseYearStr)

        if self.ccfcatHard:
            if cat.lower() == 'tv':
                searchList = [('tv', cntitle), ('tv', cuttitle)]
            elif cat.lower() == 'movie':
                searchList = [('movie', cntitle), ('movie', cuttitle)]
        else:
            if self.season:
                searchList = self.selectOrder(cntitle, cuttitle, [('tv', cntitle), ('tv', cuttitle), ('multi', cntitle)])
            elif cat.lower() == 'tv':
                searchList = self.selectOrder(cntitle, cuttitle, [('multi', cntitle), ('tv', cuttitle), ('multi', cuttitle)])
            elif cat.lower() == 'hdtv':
                searchList = [('multi', cntitle), ('multi', cuttitle)]
            elif cat.lower() == 'movie':
                searchList = self.selectOrder(cntitle, cuttitle, [('movie', cntitle), ('multi', cntitle), ('movie', cuttitle), ('multi', cuttitle)])
            else:
                searchList = [('multi', cntitle), ('multi', cuttitle)]

        for s in searchList:
            if s[0] == 'tv' and s[1]:
                print('Search TV: ' + s[1])
                # tv = TV()
                # results = tv.search(s[1])
                search = Search()
                results = search.tv_shows({"query": s[1], "year": str(intyear), "page": 1})
                if len(results) > 0:
                    if intyear > 0:
                        if self.season and 'S01' not in self.season:
                            intyear = 0
                    result = self.findYearMatch(results, intyear, strict=True)
                    if result:
                        self.saveTmdbTVResultMatch(result)
                        return self.tmdbid, self.title, self.year
                    else:
                        result = self.findYearMatch(results, intyear, strict=False)
                        if result:
                            self.saveTmdbTVResultMatch(result)
                            return self.tmdbid, self.title, self.year

            elif s[0] == 'movie' and s[1]:
                print('Search Movie:  %s (%d)' % (s[1], intyear))
                search = Search()
                if intyear == 0:
                    results = search.movies({"query": s[1], "page": 1})
                else:
                    results = search.movies({"query": s[1], "year": str(intyear), "page": 1})

                if len(results) > 0:
                    result = self.findYearMatch(results, intyear, strict=True)
                    if result:
                        self.saveTmdbMovieResult(result)
                        return self.tmdbid, self.title, self.year
                    else:
                        result = self.findYearMatch(results, intyear, strict=False)
                        if result:
                            self.saveTmdbMovieResult(result)
                            return self.tmdbid, self.title, self.year
                elif intyear > 0:
                    results = search.movies({"query": s[1], "page": 1})
                    if len(results) > 0:
                        result = self.findYearMatch(results, intyear, strict=False)
                        if result:
                            self.saveTmdbMovieResult(result)
                            return self.tmdbid, self.title, self.year
            elif s[0] == 'multi' and s[1]:
                print('Search Multi:  %s (%d)' % (s[1], intyear))
                search = Search()
                if intyear == 0:
                    results = search.multi({"query": s[1], "page": 1})
                else:
                    results = search.multi({"query": s[1], "year": str(intyear), "page": 1})

                if len(results) > 0:
                    result = self.findYearMatch(results, intyear, strict=True)
                    if result:
                        self.saveTmdbMultiResult(result)
                        return self.tmdbid, self.title, self.year
                    else:
                        result = self.findYearMatch(results, intyear, strict=False)
                        if result:
                            self.saveTmdbMultiResult(result)
                            return self.tmdbid, self.title, self.year
                elif intyear > 0:
                    results = search.multi({"query": s[1], "page": 1})
                    if len(results) > 0:
                        result = self.findYearMatch(results, intyear, strict=True)
                        if result:
                            self.saveTmdbMultiResult(result)
                            return self.tmdbid, self.title, self.year
                        else:
                            result = self.findYearMatch(results, intyear, strict=False)
                            if result:
                                self.saveTmdbMultiResult(result)
                                return self.tmdbid, self.title, self.year

        print('\033[31mTMDb Not found: [%s] [%s]\033[0m ' % (title, cntitle))
        return 0, title, intyear


    # TODO: to be continue
    def checkNameContainsId(self, torname):
        m = re.search(r'\[imdb(id)?\=(tt\d+)\]', torname, flags=re.A | re.I)
        if m:
            tmdbid, title, year = self.searchTMDbByIMDbId(m[2])
            if tmdbid > 0:
                return True
        m = re.search(r'\[tmdb(id)?\=(\d+)\]', torname, flags=re.A | re.I)
        if m:
            tmdbid, title, year = self.searchTMDbByTMDbId(self.tmdbcat, m[2])
            if tmdbid > 0:
                return True
        return False

    def searchTMDbByIMDbId(self, imdbid):
        f = Find(self.tmdb)
        print("Search : " + imdbid)
        t = f.find_by_imdb_id(imdb_id=imdbid)
        if t:
            # print(t)
# (Pdb) p t.movie_results
# [{'adult': False, 'backdrop_path': '/rcmjVmKBKONXk2LCe7GOAIHaIAO.jpg', 'id': 1068249, 'title': 'Reborn Rich', 'original_language': 'ko', 'original_title': '재벌집 막내아들', 'overview': '...', 'poster_path': '/xVtekQdaJ00cQqK2oyVJg5P7a6H.jpg', 'media_type': 'movie', 'genre_ids': [18, 14], 'popularity': 1.4, 'release_date': '2022-11-18', 'video': False, 'vote_average': 0.0, 'vote_count': 0}]
# (Pdb) t.tv_results
# [{'adult': False, 'backdrop_path': '/jG8mKDxe0LIDFBPB8uCeYGSBWCH.jpg', 'id': 153496, 'name': 'Reborn Rich', 'original_language': 'ko', 'original_name': '재벌집 막내아들', 'overview': '....', 'poster_path': '/ioywelRYOfNJ5w8aNQ5ttJo9dk1.jpg', 'media_type': 'tv', 'genre_ids': [18, 10765], 'popularity': 70.232, 'first_air_date': '2022-11-18', 'vote_average': 8.094, 'vote_count': 32, 'origin_country': ['KR']}]
            if self.tmdbcat == 'movie': 
                if t.movie_results:
                    self.tmdbcat = "movie"
                    r = t['movie_results'][0]
                    self.saveTmdbMovieResult(r)
                elif t.tv_results:
                    self.tmdbcat = "tv"
                    r = t['tv_results'][0]
                    self.saveTmdbTVResultMatch(r)
                else:
                    pass
            elif self.tmdbcat == "tv":
                if t.tv_results:
                    self.tmdbcat = "tv"
                    r = t['tv_results'][0]
                    self.saveTmdbTVResultMatch(r)
                elif t.movie_results:
                    self.tmdbcat = "movie"
                    r = t['movie_results'][0]
                    self.saveTmdbMovieResult(r)
                else:
                    pass

        return self.tmdbid, self.title, self.year


    def searchTMDbByTMDbIdTv(self, tmdbid):
        tv = TV(self.tmdb)
        print("Search tmdbid in TV: " + tmdbid)
        try:
            t = tv.details(tmdbid)
            if t:
                self.saveTmdbTVResultMatch(t)
        except:
            pass
        return self.tmdbid, self.title, self.year 

    def searchTMDbByTMDbIdMovie(self, tmdbid):
        movie = Movie(self.tmdb)
        print("Search tmdbid in Movie: " + tmdbid)
        try:
            m = movie.details(tmdbid)
            if m:
                self.saveTmdbMovieResult(m)
        except:
            pass
        return self.tmdbid, self.title, self.year 

    def searchTMDbByTMDbId(self, cat, tmdbid):
        if cat == 'tv':
            return self.searchTMDbByTMDbIdTv(tmdbid)
        elif cat == 'movie':
            return self.searchTMDbByTMDbIdMovie(tmdbid)
        else:
            self.searchTMDbByTMDbIdTv(tmdbid)
            if self.tmdbid <= 0:
                return self.searchTMDbByTMDbIdMovie(tmdbid)

        return self.tmdbid, self.title, self.year 



import re
import sys
import json
import string
import argparse
from pprint import pprint
from torcp.torcategory import cutExt


def isFullAscii(str):
    return re.fullmatch(r'[\x00-\x7F]*', str, re.A)


def containsCJK(str):
    return re.search(r'[\u4e00-\u9fa5\u3041-\u30fc]', str)


def containdCJKKeyword(str):
    return re.search(r'^(.迪士尼\b)', str)


def notTitle(str):
    return re.search(r'^(BDMV|1080[pi]|MOVIE|DISC|Vol)', str, re.A | re.I)


def cutAKA(titlestr):
    m = re.search(r'\s(/|AKA)\s', titlestr, re.I)
    if m:
        titlestr = titlestr.split(m.group(0))[0]
    return titlestr.strip()


def cutAKAJP(titlestr):
    m = re.search(r'(/|\bAKA\b)', titlestr, re.I)
    if m:
        titlestr = titlestr.split(m.group(0))[0]
    return titlestr.strip()


def getIndexItem(items, index):
    if index >= 0 and index < len(items):
        return items[index]
    else:
        return ''


def is0DayName(itemstr):
    # CoComelon.S03.1080p.NF.WEB-DL.DDP2.0.H.264-NPMS
    m = re.match(r'^\w+.*\b(BluRay|Blu-?ray|720p|1080[pi]|[xh].?26\d|2160p|576i|WEB-DL|DVD|WEBRip|HDTV)\b.*', itemstr, flags=re.A | re.I)
    return m


def getNoBracketedStr(torName, items):
    ss = torName
    for s in items:
        ss = ss.replace('[' + s + ']', '')
    ss = ss.replace('[', '')
    ss = ss.replace(']', '')
    ss = ss.strip()

    return ss


def cutBracketedTail(sstr):
    m = re.search(r'^\w+.*(\[[^]]*\]?)', sstr)
    if m:
        sstr = sstr[:m.span(1)[0]]
    return sstr


def bracketToBlank(sstr):
    dilimers = ['(', ')', '-', '–', '_', '+']
    for dchar in dilimers:
        sstr = sstr.replace(dchar, ' ')
    return re.sub(r' +', ' ', sstr).strip()


def delimerToBlank(sstr):
    dilimers = ['[', ']', '.', '{', '}', '_', ',']
    for dchar in dilimers:
        sstr = sstr.replace(dchar, ' ')
    return sstr


def cutspan(sstr, ifrom, ito):
    if (ifrom >= 0) and (len(sstr) > ito):
        sstr = sstr[0:ifrom:] + sstr[ito::]
    return sstr


def subEpisodeChar(partStr):
    partOrder = string.ascii_uppercase.find(partStr[0].upper()) + 1
    return ('Part %d' % partOrder) if partOrder > 0 else ''


def subEpisodePart(partStr):
    partOrder = string.digits.find(partStr[0])
    if partOrder < 0:
        partOrder = string.ascii_uppercase.find(partStr[0].upper()) + 1
    return ('Part %d' % partOrder) if partOrder > 0 else ''


class TorTitle:

    def parseSeason(self, sstr):
        seasonstr = ''
        seasonspan = [-1, -1]
        episodestr = ''

        # m1 = None
        # for m1 in re.finditer(r'(\bS\d+(-S\d+)?)\b', sstr, flags=re.A | re.I):
        #     pass
        m1 = re.search(r'(\bS\d+(-S?\d+))\s(?!.*\bS\d+)', sstr, flags=re.A | re.I)
        if m1:
            seasonstr = m1.group(1).strip()
            seasonspan = m1.span(1)
            sstr = sstr.replace(seasonstr, '')
            return seasonstr, seasonspan, episodestr

        # m2 = re.search(r'(\b(S\d+)([\. ]?(\d{4}[\s\.])?Ep?\d+)?)\b(?!.*S\d+)', sstr, flags=re.A | re.I)
        m2 = re.search(r'(\b(S\d+)([\. ]?(\d{4}[\s\.])?Ep?\d+)?)([A-Z]?)\b', sstr, flags=re.A | re.I)
        if m2:
            seasonstr = m2.group(1)
            seasonspan = m2.span(1)
            if m2.group(3):
                seasonstr = m2.group(2).strip()
                episodestr = m2.group(3).strip()
            if m2.group(5):
                self.subEpisode  = subEpisodeChar(m2.group(5).strip())
            return seasonstr, seasonspan, episodestr

            # seasonsapn = mcns.span(1)
            # sstr = sstr.replace(mcns.group(1), '')
        mep = re.search(r'(Ep?\d+(-Ep?\d+)?)\b', sstr, flags=re.A | re.I)
        if mep:
            seasonstr = 'S01'
            episodestr = mep.group(1).strip()
            seasonspan = mep.span(1)
            # if mep.group(2):
            #     seasonstr = mep.group(2)
            #     seasonspan = mep.span(2)
            return seasonstr, seasonspan, episodestr

        mcns = re.search(r'(第?\s*((\d+)|([一二三四五六七八九十]))(-\d+)?季)(\s*第\s*((\d+)|([一二三四五六七八九十]))集)?', sstr, flags=re.I)
        if mcns:
            # origin_seasonstr = mcns.group(1)
            seasonspan = mcns.span(1)
            ssi = mcns.group(2)
            iss = '一二三四五六七八九'.find(ssi)
            if iss >= 0:
                ssi = str(iss+1).zfill(2)
            seasonstr = 'S' + ssi
            if mcns.group(6):
                episodestr = 'E' + mcns.group(7).strip()
            return seasonstr, seasonspan, episodestr
        return seasonstr, seasonspan, episodestr


    def parseYear(self, sstr):
        yearstr = ''
        yearspan = [-1, -1]
        m2 = re.search(
            r'\b((19\d{2}\b|20\d{2})(-19\d{2}|-20\d{2})?)\b(?!.*\b\d{4}\b.*)',
            sstr,
            flags=re.A | re.I)
        if m2:
            yearstr = m2.group(1).strip()
            yearspan = m2.span(1)
            if re.search(r'[\(\[\{]' + yearstr+r'\b', sstr):
                # sstr = sstr[:yearspan[0] - 1]
                yearspan = [yearspan[0]-1, yearspan[1]+1]
            # elif re.search(r'\w.*' + yearstr+r'\b', sstr):
            #     sstr = sstr[:yearspan[0]]
        return yearstr, yearspan


    def parseJpAniName(self, torName):
        yearstr, yearspan = self.parseYear(torName)

        items = re.findall(r'\[([^]]*[^[]*)\]', torName)

        if len(items) < 2:
            return self.parse0DayMovieName(torName)

        for s in items:
            if is0DayName(s):
                return self.parse0DayMovieName(s)

        strLeft = getNoBracketedStr(torName, items)
        if len(strLeft) > 0:
            # yearstr, titlestr = getYearStr(torName)
            titlestr = bracketToBlank(strLeft)
            return cutAKAJP(titlestr), yearstr, '', '', ''

        jptitles = []
        titlestrs = []
        jptitle = ''
        titlestr = ''
        for item in items:
            if re.match(r'^(BDMV|EAC|XLD|1080[pi]|MOVIE|DISC|Vol\.?\d+|MPEG|合集|ALBUM|SBCV|FLAC|SINGLE|V\.A|VVCL)\b', item, re.A | re.I):
                continue
            if re.match(r'^\d+$', item):
                continue

            if containsCJK(item):
                jptitles.append(item)
            else:
                titlestrs.append(item)

        if len(titlestrs) > 0:
            titlestr = titlestrs[0]
            # titlestr = max(titlestrs, key=len)
            if jptitles:
                jptitle = max(jptitles, key=len)
        else:
            if jptitles:
                # jptitle = jptitles[0]
                jptitle = max(jptitles, key=len)
                titlestr = jptitle
            else:
                pass
                # raise 'Some thing Wrong'

        titlestr = cutBracketedTail(titlestr)
        titlestr = bracketToBlank(titlestr)

        return cutAKAJP(titlestr), yearstr, '', '', jptitle

    def checkAfterSeason(self, sstr, seasonspan):
        if self.subEpisode:
            return sstr
        afterStr = sstr[seasonspan[1]:]
        m = re.search(r'part\s*(\d+|[A-Z])', afterStr, flags=re.I)
        if m:
            self.subEpisode = subEpisodePart(m.group(1))
            return cutspan(sstr, seasonspan[0], seasonspan[1]+m.span(0)[1])
        else:
            return sstr

    def parse0DayMovieName(self, torName):
        sstr = cutExt(torName)

        failsafeTitle = sstr

        sstr = re.sub(r'\W(Disney|DSNP|Hami|ATVP|Netflix|NF|KKTV|Amazon|AMZN|HMAX|Friday|\d+fps)\W*WEB-?DL.*$', '', sstr, flags=re.I)
        sstr = re.sub(
            r'\b((UHD)?\s+BluRay|Blu-?ray|720p|1080[pi]|2160p|576i|WEB-DL|\.DVD\.|UHD|WEBRip|HDTV|Director(\'s)?[ .]Cut|REMASTERED|LIMITED|Complete(?=[. -]\d+)|SUBBED|TV Series).*$',
            '',
            sstr,
            flags=re.I)
        sstr = re.sub(r'\[Vol.*\]$', '', sstr, flags=re.I)

        sstr = re.sub(r'\W?(IMAX|Extended Cut|\d+CD|APE整轨)\b.*$', '', sstr, flags=re.I)
        sstr = re.sub(r'[\[\(](BD\d+|WAV\d*|(CD\-)?FLAC|Live|DSD\s?\d*)\b.*$', '', sstr, flags=re.I)
        sstr = re.sub(r'^\W?(BDMV|\BDRemux|\bCCTV\d(HD|K)?|BD\-?\d*|[A-Z]{1,5}TV)\W*', '', sstr, flags=re.I)

        sstr = re.sub(r'\{[^\}]*\}.*$', '', sstr, flags=re.I)
        sstr = re.sub(r'([\s\.-](\d+)?CD[\.-]WEB|[\s\.-](\d+)?CD[\.-]FLAC|[\s\.-][\[\(\{]FLAC[\]\)\}]).*$', '', sstr, flags=re.I)
        sstr = re.sub(r'\bFLAC\b.*$', '', sstr, flags=re.I)
        sstr = re.sub(r'^[\[\(]\d+[^\)\]]*[\)\]]', '', sstr, flags=re.I)

        sstr = re.sub(r'^\W?CC_?\b', '', sstr, flags=re.I)
        if sstr and sstr[-1] in ['(', '[', '{']:
            sstr = sstr[:-1]
        sstr = delimerToBlank(sstr)
        if sstr:
            failsafeTitle = sstr

        seasonstr, seasonspan, episodestr = self.parseSeason(sstr)
        if not seasonstr:
            seasonstr, seasonspan2, episodestr = self.parseSeason(torName)
        self.seasonstr = seasonstr
        sstr = self.checkAfterSeason(sstr, seasonspan)
        
        yearstr, yearspan = self.parseYear(sstr)
        if not yearstr:
            yearstr, yearspan = self.parseYear(torName)
            yearspan = [-1, -1]

        if seasonspan[0] > yearspan[0]:
            syspan = seasonspan
            systr = seasonstr
        else:
            syspan = yearspan
            systr = yearstr

        skipcut = False
        if syspan and syspan[0] > 1 :
            spanstrs = sstr.split(systr)
            if containdCJKKeyword(sstr[:syspan[0]]):
                sstr = sstr[syspan[1]:]
                skipcut = True
            else:
                sstr = sstr[:syspan[0]]

        if not skipcut:
            sstr = cutspan(sstr, seasonspan[0], seasonspan[1])
            sstr = cutspan(sstr, yearspan[0], yearspan[1])
        if sstr:
            failsafeTitle = sstr
        sstr = re.sub(r'\b(Theatrical|Extended)\s+Version', '', sstr, flags=re.I)
        sstr = re.sub(r'\b(Classic|Unrated)\s*$', '', sstr, flags=re.I)
        sstr = re.sub(r'\b\w+(影|场|念|港|修复)版', '', sstr, flags=re.I)
        sstr = re.sub(r'(\b剧集|\b全\d+集|\b\d+集全|\b\w+(影|场|念|港)版|\b国语|\bDis[kc]\s*\d*|\bBD\d*).*$', '', sstr, flags=re.I)
        sstr = re.sub(r'(粤语|DIY|中字)[\u4e00-\u9fa5\u3041-\u30fc ]*$', '', sstr, flags=re.I)
        if sstr and sstr[-1] in ['(', '[', '{', '（', '【']:
            sstr = sstr[:-1]

        sstr = re.sub(r'(?<!^)(Ep?\d+(-Ep?\d+)?)\s*$', '', sstr, flags=re.A | re.I)

        # if titlestr.endswith(')'):
        #     titlestr = re.sub(r'\(.*$', '', sstr).strip()
        cntitle = ''
        if containsCJK(sstr):
            cntitle = sstr
            # m = re.search(r'^.*[^\x00-\x7F](S\d+|\s|\.|\d|-|\))*\b(?=[a-zA-Z])', sstr, flags=re.A)
            # m = re.search( r'^.*[^a-zA-Z_\- &0-9](S\d+|\s|\.|\d|-)*\b(?=[A-Z])', titlestr, flags=re.A)
            m = re.search(r'^.*[一-鿆\?？]\s*(S\d+(-S\d+)?|\([^(]*\)|\d+-\d+)?[： ]*(?=[0-9a-zA-Z])',
                            sstr, flags=re.A)
            if m:
                # ['(', ')', '-', '–', '_', '+']
                cntitle = m.group(0)
                if not re.search(r'\s[\-\+]\s', cntitle):
                    # if len(sstr)-len(cntitle) > 4:
                    sstr = sstr.replace(cntitle, '')
            else:
                m = re.search(r'^([\w\s]+)\s([一-鿆\?？]+)\s*$', sstr, flags=re.A)
                if m:
                    cntitle = m.group(1)
                    if not re.search(r'\s[\-\+]\s', cntitle):
                        sstr = sstr.replace(cntitle, '')
            cntitle = re.sub(r' +', ' ', cntitle).strip()

        titlestr = bracketToBlank(sstr)
        titlestr = cutAKA(titlestr)
        if not containsCJK(titlestr) and len(titlestr) < 3:
            titlestr = bracketToBlank(failsafeTitle)

        return titlestr, yearstr, seasonstr, episodestr, cntitle


    def parseMovieName(self, torName):
        if torName.startswith('[') and torName.endswith('SP'):
            m = re.search(r'\]([^]]*\+.?SP)$', torName, flags=re.I)
            if m:
                namestr = torName[:m.span(1)[0]]
                return self.parseJpAniName(namestr)

        if torName.startswith('[') and torName.endswith(']'):
            return self.parseJpAniName(torName)
        else:
            return self.parse0DayMovieName(torName)

    def to_json(self):
        return {
            'title': self.title,
            'year': self.yearstr,
            'season': self.season,
            'episode': self.episode,
            'cntitle': self.cntitle
        }
    
    def to_csv(self, delimiter=','):
        return delimiter.join([self.title, self.yearstr, self.season, self.episode, self.cntitle])

    def __init__(self, torName):
        self.subEpisode = ''
        self.seasonstr = ''
        self.title, self.yearstr, self.season, self.episode, self.cntitle = self.parseMovieName(torName)


def main():
    parser = argparse.ArgumentParser(
        description='torcp: a script to parse torrent name.'
    )
    parser.add_argument('TORRENT_NAME', nargs="+", help='The torrent name.')
    parser.add_argument('-f', '--format', default='json', help='Output format, accept json,csv')
    parser.add_argument('-P', '--pretty-print', default=False, help='Pretty print json output')

    args = parser.parse_args()
    tortitles = [TorTitle(t) for t in args.TORRENT_NAME]
    if tortitles and args.format == 'csv':
        print(','.join(tortitles[0].to_json().keys()))
        for t in tortitles:
            print(t.to_csv())
    else:
        if args.pretty_print:
            pprint([t.to_json() for t in tortitles])
        else:
            print(json.dumps([t.to_json() for t in tortitles], ensure_ascii=False))
    
if __name__ == '__main__':
    main()



import os
import json


class CacheManager:
    def __init__(self, logdir):
        self.search_history_file_path = os.path.join(logdir, 'CACHE_LIST.json')

    def openCache(self):
        try:
            with open(self.search_history_file_path, 'r',
                      encoding='utf8') as f:
                self.search_history = json.load(f)
            self.history_json_fd = open(self.search_history_file_path,
                                        'r+',
                                        encoding='utf8')
            return True
        except:
            self.search_history = {'path_dupped': [], 'basename': []}
            self.history_json_fd = open(self.search_history_file_path,
                                        'w',
                                        encoding='utf8')
            json.dump(self.search_history, self.history_json_fd, indent=4)
            # self.history_json_fd.close()
            return False

    def isCached(self, file_path):
        for name in self.search_history['path_dupped']:
            if file_path == name:
                return True
        return False

    def append(self, file_path):
        # append file_path to history
        if file_path not in self.search_history['path_dupped']:
            self.search_history['path_dupped'].append(file_path)

        json.dump(self.search_history, self.history_json_fd, indent=4)
        self.history_json_fd.seek(0)

    def closeCache(self):
        self.history_json_fd.close()




# -*- coding: utf-8 -*-
"""
A script hardlink media files and directories in Emby-happy naming and structs.
"""
#  Usage:
#   python3 torcp.py -h
#
#  Example: hard link to a seperate dir
#    python3 torcp.py /home/ccf2012/Downloads/  -d=/home/ccf2012/emby/
#
#
import re
import os
import time
import datetime
import argparse
import shutil
import glob
import platform
import codecs

from torcp.tmdbparser import TMDbNameParser
from torcp.torcategory import TorCategory
from torcp.tortitle import TorTitle, is0DayName
from torcp.cacheman import CacheManager

ARGS = None

CATNAME_TV = 'TV'
CATNAME_MOVIE = 'Movie'


def ensureDir(file_path):
    if os.path.isfile(file_path):
        file_path = os.path.dirname(file_path)
    if not os.path.exists(file_path):
        os.makedirs(file_path)


def makeLogfile(fromLoc, toLocPath, logDir=None):
    if not ARGS.make_log:
        return 
    destDir = os.path.join(ARGS.hd_path, toLocPath)
    if not logDir:
        fromLocDir = fromLoc
        if os.path.isfile(fromLoc):
            fromLocDir = os.path.dirname(fromLoc)
            if re.search(r'\bS\d+$', fromLocDir):
                fromLocDir = os.path.dirname(fromLocDir)
        originName, ext = os.path.splitext(os.path.basename(fromLocDir))
    else:
        originName = os.path.basename(logDir)
    if not originName:
        originName = '_'
    logFilename = os.path.join(destDir, originName + '.log')
    
    if not os.path.exists(destDir):
        ensureDir(destDir)
    with codecs.open(logFilename, "a+", "utf-8") as logfile:
        logfile.write(fromLoc+'\n')
        logfile.close()


def hdlinkCopy(fromLoc, toLocPath, toLocFile=''):
    if os.path.islink(fromLoc):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % fromLoc)
        return
    destDir = os.path.join(ARGS.hd_path, toLocPath)
    if not ARGS.dryrun:
        ensureDir(destDir)
        makeLogfile(fromLoc, toLocPath)
    if os.path.isfile(fromLoc):
        if toLocFile:
            destFile = os.path.join(destDir, toLocFile)
        else:
            destFile = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destFile):
            if ARGS.dryrun:
                print(fromLoc, ' ==> ', destFile)
            else:
                print('ln ', fromLoc, destFile)
                os.link(fromLoc, destFile)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destFile)

    elif os.path.isdir(fromLoc):
        destDir = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destDir):
            if ARGS.dryrun:
                print('copytree' + fromLoc + ' ==> ' + destDir)
            else:
                print('copytree ', fromLoc, destDir)
                shutil.copytree(fromLoc, destDir, copy_function=os.link)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destDir)
    else:
        print('File/Dir %s not found' % (fromLoc))


def pathMove(fromLoc, toLocFolder, toLocFile=''):
    if os.path.islink(fromLoc):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % fromLoc)
        return
    destDir = os.path.join(ARGS.hd_path, toLocFolder)
    if not ARGS.dryrun:
        ensureDir(destDir)
        makeLogfile(fromLoc, toLocFolder)
        if ARGS.sleep:
            time.sleep(ARGS.sleep)
    if os.path.isfile(fromLoc):
        if toLocFile:
            destFile = os.path.join(destDir, toLocFile)
        else:
            destFile = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destFile):
            if ARGS.dryrun:
                print(fromLoc, ' ==> ', destFile)
            else:
                print('mv ', fromLoc, destFile)
                os.rename(fromLoc, destFile)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destFile)
    else:
        destDir = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destDir):
            if ARGS.dryrun:
                print(fromLoc, ' ==> ', destDir)
            else:
                print('mvdir ', fromLoc, destDir)
                shutil.move(fromLoc, destDir)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destDir)


def symbolLink(fromLoc, toLocPath, toLocFile=''):
    if os.path.islink(fromLoc):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % fromLoc)
        return
    destDir = os.path.join(ARGS.hd_path, toLocPath)
    if not ARGS.dryrun:
        ensureDir(destDir)
        makeLogfile(fromLoc, toLocPath)
    if os.path.isfile(fromLoc):
        if toLocFile:
            destFile = os.path.join(destDir, toLocFile)
        else:
            destFile = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destFile):
            if ARGS.dryrun:
                print(fromLoc, ' ==> ', destFile)
            else:
                print('ln -s', fromLoc, destFile)
                os.symlink(fromLoc, destFile)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destFile)

    elif os.path.isdir(fromLoc):
        destDir = os.path.join(destDir, os.path.basename(fromLoc))
        if not os.path.exists(destDir):
            if ARGS.dryrun:
                print('(DIR) ln -s ' + fromLoc + ' ==> ' + destDir)
            else:
                print('(DIR) ln -s ', fromLoc, destDir)
                os.symlink(fromLoc, destDir)
                # shutil.copytree(fromLoc, destDir, copy_function=os.link)
        else:
            print('\033[32mTarget Exists: [%s]\033[0m ' % destDir)
    else:
        print('File/Dir %s not found' % (fromLoc))


def hdlinkLs(loc):
    destDir = os.path.join(ARGS.hd_path, loc)
    ensureDir(destDir)
    return os.listdir(destDir)


def targetCopy(fromLoc, toLocPath, toLocFile=''):
    if os.path.islink(fromLoc):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % fromLoc)
        return

    if ARGS.move_run:
        pathMove(fromLoc, toLocPath, toLocFile)
    else:
        if ARGS.symbolink:
            symbolLink(fromLoc, toLocPath, toLocFile)
        else:
            hdlinkCopy(fromLoc, toLocPath, toLocFile)


def getSeasonFromFolderName(folderName, failDir=''):
    m1 = re.search(r'(\bS\d+(-S\d+)?|第(\d+)季)', folderName, flags=re.A | re.I)
    if m1:
        if m1.group(3):
            return 'S' + m1.group(3)
        else:
            return m1.group(1)
    else:
        return folderName
        # return failDir


def fixNtName(file_path):
    # file_path = re.sub(r'(\:|\?|<|>|\*|\\|\")', ' ', file_path)
    if platform.system() == 'Windows':
        file_path = re.sub(r'(\:|\?|<|>|\*|\\|/|\")', ' ', file_path)
    else:
        file_path = re.sub(r'/', ' ', file_path)
    return file_path


def genMediaFolderName(nameParser):
    if nameParser.tmdbid > 0:
        if ARGS.emby_bracket:
            tmdbTail = '[tmdbid=' + str(nameParser.tmdbid) + ']'
        elif ARGS.plex_bracket:
            tmdbTail = '{tmdb-' + str(nameParser.tmdbid) + '}'
        else:
            tmdbTail = ''

        subdir_title = nameParser.title

        if ARGS.lang:
            if ARGS.lang.lower() == 'all':
                subdir_title = os.path.join(nameParser.original_language,
                                            nameParser.title)
            else:
                ollist = ARGS.lang.lower().split(',')
                if nameParser.original_language in ollist:
                    subdir_title = os.path.join(nameParser.original_language,
                                                nameParser.title)
                else:
                    subdir_title = os.path.join('other', nameParser.title)

        if nameParser.year > 0:
            mediaFolderName = '%s (%d) %s' % (
                subdir_title, nameParser.year, tmdbTail)
        else:
            mediaFolderName = '%s %s' % (subdir_title, tmdbTail)

    else:
        if nameParser.ccfcat == 'tv':
            # if not nameParser.season:
            #     tempseason = 'S01'
            if nameParser.year > 0 and nameParser.season == 'S01':
                mediaFolderName = '%s (%d)' % (nameParser.title,
                                               nameParser.year)
            else:
                mediaFolderName = nameParser.title
        else:
            if nameParser.year > 0:
                mediaFolderName = '%s (%d)' % (nameParser.title,
                                               nameParser.year)
                # mediaFolderName = nameParser.title + ' (' + str(
                #     nameParser.year) + ')'
            else:
                mediaFolderName = nameParser.title
            
    return mediaFolderName.strip()


def isMediaFileType(file_ext):
    return KEEPEXTALL or file_ext.lower() in KEEPEXTS


def copyTVSeasonItems(tvSourceFullPath, tvFolder, seasonFolder, groupName,
                      resolution):
    if os.path.isdir(os.path.join(tvSourceFullPath, 'BDMV')):
        # break, process BDMV dir for this dir
        bdmvTVFolder = os.path.join(tvFolder, seasonFolder)
        processBDMV(tvSourceFullPath, bdmvTVFolder, CATNAME_TV)
        return

    # catutil = TorCategory()
    for tv2item in os.listdir(tvSourceFullPath):
        tv2itemPath = os.path.join(tvSourceFullPath, tv2item)
        if os.path.isdir(tv2itemPath):
            print('\033[31mSKIP dir in TV: [%s]\033[0m ' % tv2itemPath)
        else:
            filename, file_ext = os.path.splitext(tv2item)
            seasonFolderFullPath = os.path.join(CATNAME_TV, tvFolder, seasonFolder)
            if isMediaFileType(file_ext):
                if ARGS.origin_name:
                    newTVFileName = os.path.basename(tv2item)
                else:
                    if not groupName:
                        tc = TorCategory(tv2item)
                        cat, groupName = tc.ccfcat, tc.group
                        if not resolution:
                            resolution = tc.resolution
                    newTVFileName = genTVSeasonEpisonGroup(
                        tv2item, groupName, resolution)
                # makeLogfile(tv2itemPath, seasonFolderFullPath, tvSourceFullPath)
                targetCopy(tv2itemPath, seasonFolderFullPath, newTVFileName)
            elif file_ext.lower() in ['.iso']:
                # TODO: aruba need iso when extract_bdmv
                if ARGS.full_bdmv or ARGS.extract_bdmv:
                    targetCopy(tv2itemPath, seasonFolderFullPath)


def uselessFile(entryName):
    return entryName in ['@eaDir', '.DS_Store', '.@__thumb']


def selfGenCategoryDir(dirName):
    return dirName in [
        'MovieEncode', 'MovieRemux', 'MovieWebdl', 'MovieBDMV', 'BDMVISO',
        CATNAME_MOVIE, CATNAME_TV, 'TMDbNotFound'
    ]


def genTVSeasonEpisonGroup(mediaFilename, groupName, resolution):
    tt = TorTitle(mediaFilename)
    tvTitle, tvYear, tvSeason, tvEpisode, cntitle = tt.title, tt.yearstr, tt.season, tt.episode, tt.cntitle
    # tvTitle = fixNtName(tvTitle)

    tvEpisode = re.sub(r'^Ep\s*', 'E', tvEpisode, flags=re.I)
    filename, file_ext = os.path.splitext(mediaFilename)
    ch1 = '- ' if (resolution or groupName) else ''
    ch2 = '_' if (resolution and groupName) else ''

    tvname = '%s %s %s%s %s%s%s' % (tvTitle,
                                    ('(' + tvYear + ')') if tvYear else '',
                                    tvSeason.upper() if tvSeason else '',
                                    tvEpisode.upper() if tvEpisode else '',
                                    (tt.subEpisode+' ') if tt.subEpisode else '',
                                    ch1+resolution if resolution else '',
                                    ch2+groupName if groupName else '')

    tvname = tvname.strip() + file_ext

    # filename, file_ext = os.path.splitext(mediaFilename)
    # ch1 = ' - ' if (resolution or groupName) else ''
    # ch2 = '_' if (resolution and groupName) else ''
    # tvname = tvTitle + ((' (' + tvYear + ')') if tvYear else '') + (
    #     ' ' + tvSeason.upper() if tvSeason else
    #     '') + (tvEpisode.upper() if tvEpisode else '') + ch1 + (
    #         resolution if resolution else '') + ch2 + (groupName if groupName
    #                                                    else '') + file_ext
    return tvname.strip()


def countMediaFile(filePath):
    types = ('*.mkv', '*.mp4', '*.ts')
    curdir = os.getcwd()
    os.chdir(filePath)
    mediaCount = 0
    for files in types:
        mediaCount += len(glob.glob(files))
    os.chdir(curdir)
    return mediaCount


def getFirstMediaFile(filePath):
    mediaFiles = getMediaFiles(filePath)
    return os.path.basename(mediaFiles[0]) if mediaFiles else None
    

def getMediaFiles(filePath):
    types = ('*.mkv', '*.mp4', '*.ts')
    filesFound = []
    curdir = os.getcwd()
    os.chdir(filePath)
    for files in types:
        filesFound.extend(glob.glob(files))
    os.chdir(curdir)
    return filesFound


def getMusicFile(filePath):
    types = ('*.flac', '*.ape', '*.wav')
    files_grabbed = []
    curdir = os.getcwd()
    os.chdir(filePath)
    for files in types:
        files_grabbed.extend(glob.glob(files))
    os.chdir(curdir)
    if files_grabbed:
        return os.path.basename(files_grabbed[0])
    else:
        return None


def fixSeasonGroupWithFilename(folderPath, folderSeason, folderGroup, folderResolution, destFolderName):
    season = folderSeason
    group = folderGroup
    resolution = folderResolution
    foldername = destFolderName
    testFile = getFirstMediaFile(folderPath)
    if testFile:
        p = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang)
        p.parse(testFile, useTMDb=False)
        if not folderGroup:
            group = p.group
        if not folderSeason:
            season = p.season
        if not season:
            season = 'S01'
    return season, group, foldername, resolution


def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
                      resolution, folderTmdbParser):
    if os.path.islink(tvSourceFolder):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % tvSourceFolder)
        return
    if os.path.isdir(os.path.join(tvSourceFolder, 'BDMV')):
        if ARGS.full_bdmv or ARGS.extract_bdmv:
            # a BDMV dir in a TV folder, treat as Movie
            processBDMV(tvSourceFolder, genFolder, 'MovieM2TS')
            targetDirHook(os.path.join('MovieM2TS', genFolder), tmdbidstr=str(folderTmdbParser.tmdbid))
        else:
            print('\033[31mSkip BDMV/ISO  %s \033[0m' % genFolder)
        return

    parseSeason, parseGroup, genFolder, resolution = fixSeasonGroupWithFilename(
        tvSourceFolder, folderSeason, groupName, resolution, genFolder)

    if not os.path.isdir(tvSourceFolder):
        return

    for tvitem in os.listdir(tvSourceFolder):
        if uselessFile(tvitem):
            print('\033[34mSKIP useless file: [%s]\033[0m ' % tvitem)
            continue
        if selfGenCategoryDir(tvitem):
            print('\033[34mSKIP self-generated dir: [%s]\033[0m ' % tvitem)
            continue

        tvitemPath = os.path.join(tvSourceFolder, tvitem)
        if os.path.isdir(tvitemPath):
            seasonFolder = getSeasonFromFolderName(tvitem, failDir=parseSeason)
            copyTVSeasonItems(tvitemPath, genFolder, seasonFolder, parseGroup,
                              resolution)
        else:
            filename, file_ext = os.path.splitext(tvitemPath)
            if isMediaFileType(file_ext):
                if ARGS.origin_name:
                    newTVFileName = os.path.basename(tvitemPath)
                else:
                    newTVFileName = genTVSeasonEpisonGroup(
                        tvitem, parseGroup, resolution)
                seasonFolderFullPath = os.path.join(CATNAME_TV, genFolder,
                                                    parseSeason)
                # makeLogfile(tvitemPath, seasonFolderFullPath, tvSourceFolder)
                targetCopy(tvitemPath, seasonFolderFullPath, newTVFileName)

    mkPlexMatch(os.path.join(CATNAME_TV, genFolder), folderTmdbParser)
    targetDirHook(os.path.join(CATNAME_TV, genFolder), tmdbidstr=str(folderTmdbParser.tmdbid))


def genMovieTMDbOriginName(mediaSrc, movieName, year, nameParser=None):
    originName = os.path.basename(mediaSrc)
    # filename, file_ext = os.path.splitext(mediaSrc)
    ch1 = ' - '
    tmdbTail = ''
    if (nameParser and nameParser.tmdbid > 0 and ARGS.filename_emby_bracket and ARGS.emby_bracket):
        tmdbTail = ' [tmdbid=' + str(nameParser.tmdbid) + ']'
    medianame = movieName + ((' (' + year + ')' ) if year else '') + tmdbTail + ch1 + originName
    return medianame.strip()


def genMovieResGroup(mediaSrc, movieName, year, resolution, group, nameParser=None):
    filename, file_ext = os.path.splitext(mediaSrc)
    ch1 = ' - ' if (resolution or group) else ''
    ch2 = '_' if (resolution and group) else ''
    tmdbTail = ''
    if (nameParser and nameParser.tmdbid > 0 and ARGS.filename_emby_bracket and ARGS.emby_bracket):
        tmdbTail = ' [tmdbid=' + str(nameParser.tmdbid) + ']'
    medianame = movieName + ((' (' + year + ')' ) if year else '') + tmdbTail + ch1 + (
        resolution if resolution else '') + ch2 + (group
                                                   if group else '') + file_ext
    return medianame.strip()


def getLargestFiles(dirName):
    fileSizeTupleList = []
    largestSize = 0
    largestFiles = []

    for i in os.listdir(dirName):
        p = os.path.join(dirName, i)
        if os.path.isfile(p):
            fileSizeTupleList.append((p, os.path.getsize(p)))

    if len(fileSizeTupleList) > 0:
        fileSizeTupleList.sort(key=lambda x: x[1], reverse=True)

        a, largestSize = fileSizeTupleList[0]
        for fileName, fileSize in fileSizeTupleList:
            if fileSize > (largestSize * 0.6):
                largestFiles.append(fileName)
        return largestFiles
    else:
        return []


def setArgsCategory():
    cat = ''
    if ARGS.tv:
        # if parser.ccfcat != 'TV':
        #     print('\033[34mWarn: I don\'t think it is TV  %s \033[0m' % parser.title)
        cat = CATNAME_TV
    elif ARGS.movie:
        # if parser.ccfcat not in ['MovieEncode', 'MovieWebdl', 'MovieRemux', 'MovieBDMV', 'MV']:
        #     print('\033[34mWarn: I don\'t think it is Movie  %s \033[0m' % parser.title)
        cat = CATNAME_MOVIE
    return cat


def genCatFolderName(parser):
    global CATNAME_MOVIE, CATNAME_TV
    if ARGS.tmdb_api_key and parser.tmdbid <= 0 and parser.tmdbcat in ['tv', 'movie']:
        return 'TMDbNotFound'
    else:
        if parser.tmdbcat == 'movie':
            CATNAME_MOVIE = ARGS.movie_folder_name
            return ARGS.movie_folder_name
        elif parser.tmdbcat == 'tv':
            CATNAME_TV = ARGS.tv_folder_name
            return ARGS.tv_folder_name
        else:
            return parser.ccfcat


def isCollections(folderName):
    return re.search(r'(\bPack$|合集|Anthology|Trilogy|Quadrilogy|Tetralogy|(?<!Criterion[ .])Collections?|国语配音4K动画电影$)',
                     folderName,
                     flags=re.I)


def processBDMV(mediaSrc, folderGenName, catFolder):
    destCatFolderName = os.path.join(catFolder, folderGenName)
    if ARGS.full_bdmv:
        for bdmvItem in os.listdir(mediaSrc):
            fullBdmvItem = os.path.join(mediaSrc, bdmvItem)
            targetCopy(fullBdmvItem, destCatFolderName)
        return

    if ARGS.extract_bdmv:
        bdmvDir = os.path.join(mediaSrc, 'BDMV', 'STREAM')
        if not os.path.isdir(bdmvDir):
            print('\033[31m BDMV/STREAM/ dir not found in   %s \033[0m' %
                  mediaSrc)
            return

        largestStreams = getLargestFiles(bdmvDir)
        for stream in largestStreams:
            # fn, ext = os.path.splitext(stream)
            tsname = os.path.basename(mediaSrc) + ' - ' + os.path.basename( stream)
            targetCopy(stream, destCatFolderName, tsname)

    else:
        print('\033[31mSkip BDMV/ISO  %s \033[0m' % mediaSrc)


def processMusic(mediaSrc, folderCat, folderGenName):
    # destCatFolderName = os.path.join(folderCat, folderGenName)
    targetCopy(mediaSrc, folderCat)
    # TODO: new item add to Music folder cause full update
    targetDirHook('Music', tmdbidstr='music')


def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
    if os.path.isdir(os.path.join(mediaSrc, 'BDMV')):
        # break, process BDMV dir for this dir
        if ARGS.full_bdmv or ARGS.extract_bdmv:
            processBDMV(mediaSrc, folderGenName, 'MovieM2TS')
            targetDirHook(os.path.join('MovieM2TS', folderGenName), tmdbidstr=str(folderTmdbParser.tmdbid))
        else:
            print('\033[31mSkip BDMV/ISO  %s \033[0m' % mediaSrc)
        return

    if not os.path.isdir(mediaSrc):
        return

    testFile = getMusicFile(mediaSrc)
    if testFile:
        processMusic(mediaSrc, 'Music', folderGenName)
        return

    countMediaFiles = countMediaFile(mediaSrc)
    for movieItem in os.listdir(mediaSrc):
        if uselessFile(movieItem):
            print('\033[34mSKIP useless file: [%s]\033[0m ' % movieItem)
            continue
        if selfGenCategoryDir(movieItem):
            print('\033[34mSKIP self-generated dir: [%s]\033[0m ' % movieItem)
            continue

        if (os.path.isdir(os.path.join(mediaSrc, movieItem))):
            # Dir in movie folder
            if os.path.isdir(os.path.join(mediaSrc, movieItem, 'BDMV')):
                processBDMV(os.path.join(mediaSrc, movieItem),
                            os.path.join(folderGenName, movieItem),
                            'MovieM2TS')
            else:
                print('\033[34mSKip dir in movie folder: [%s]\033[0m ' %
                      movieItem)
            continue
        
        filename, file_ext = os.path.splitext(movieItem)
        if file_ext.lower() in ['.iso']:
            # TODO: aruba need iso when extract_bdmv
            if ARGS.full_bdmv or ARGS.extract_bdmv:
                destCatFolderName = os.path.join('BDMVISO', folderGenName)
                targetCopy(os.path.join(mediaSrc, movieItem),
                           destCatFolderName)
                targetDirHook(destCatFolderName, tmdbidstr='iso') 
            else:
                print('\033[31mSKip iso file: [%s]\033[0m ' % movieItem)
            continue

        if not isMediaFileType(file_ext):
            print('\033[34mSkip : %s \033[0m' % movieItem)
            continue

        p = folderTmdbParser
        if (folderTmdbParser.tmdbid <= 0) or countMediaFiles > 1:
            fnok = is0DayName(movieItem)
            if fnok:
                pf = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang,
                                    ccfcat_hard=setArgsCategory())
                pf.parse(movieItem, useTMDb=(ARGS.tmdb_api_key is not None))
                pf.title = fixNtName(pf.title)
                if pf.tmdbid > 0 or fnok:
                    p = pf

        cat = genCatFolderName(p)
        destFolderName = genMediaFolderName(p)
        destCatFolderName = os.path.join(cat, destFolderName)

        if cat == CATNAME_TV:
            print('\033[31mMiss Categoried TV: [%s]\033[0m ' % mediaSrc)
            copyTVFolderItems(mediaSrc, destFolderName, p.season, p.group,
                              p.resolution, p)
            # parseSeason = fixSeasonName(parseSeason)
            # if cat != folderCat:
            #     copyTVFolderItems(mediaSrc, destFolderName, p.season, p.group,
            #                       p.resolution)
            # else:
            #     copyTVFolderItems(mediaSrc, folderGenName, p.season, p.group,
            #                       p.resolution)
            return
        elif cat in ['TMDbNotFound', 'HDTV', 'Audio', 'eBook']:
            targetCopy(mediaSrc, cat)
            targetDirHook(cat, tmdbidstr=str(p.tmdbid))
            continue
        else:
            if ARGS.origin_name:
                newMovieName = os.path.basename(movieItem)
            elif ARGS.tmdb_origin_name:
                yearstr = str(p.year) if p.year > 0 else ''
                newMovieName = genMovieTMDbOriginName(movieItem, p.title, yearstr,
                                                nameParser=p)
            else:
                yearstr = str(p.year) if p.year > 0 else ''
                newMovieName = genMovieResGroup(movieItem, p.title, yearstr,
                                                p.resolution, p.group, nameParser=p)
            mediaSrcItem = os.path.join(mediaSrc, movieItem)
            # makeLogfile(mediaSrcItem, destCatFolderName)
            targetCopy(mediaSrcItem, destCatFolderName, newMovieName)
            targetDirHook(destCatFolderName, tmdbidstr=str(p.tmdbid))


def mkPlexMatch(targetDir, tmdbParser):
    if not ARGS.make_plex_match:
        return

    if not tmdbParser:
        return

    pmfilepath = os.path.join(ARGS.hd_path, targetDir, '.plexmatch')
    with open(pmfilepath, "w") as pmfile:
        pmfile.write("Title: %s\ntmdbid: %d\n" %
                     (tmdbParser.title, tmdbParser.tmdbid))
        if tmdbParser.year > 1990:
            pmfile.write("Year: %d\n" % (tmdbParser.year))


def targetDirHook(targetDir, tmdbidstr=''):
    # exportTargetDir = os.path.join(ARGS.hd_path, targetDir)
    exportTargetDir = targetDir
    print('Target Dir: ' + exportTargetDir)
    if ARGS.after_copy_script:
        import subprocess        
        cmd = [ARGS.after_copy_script, exportTargetDir, CUR_MEDIA_NAME, str(tmdbidstr)]
        subprocess.Popen(cmd).wait()
        # os.system("%s %s" % (ARGS.next_script, targetDir))
    return


def processOneDirItem(cpLocation, itemName, imdbidstr='', tmdbidstr=''):
    global CUR_MEDIA_NAME 
    CUR_MEDIA_NAME = itemName
    mediaSrc = os.path.join(cpLocation, itemName)
    if os.path.islink(mediaSrc):
        print('\033[31mSKIP symbolic link: [%s]\033[0m ' % mediaSrc)
        return

    print(" >> [%s] %s %s" % (itemName, imdbidstr, tmdbidstr))
    cat = setArgsCategory()
    p = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang, ccfcat_hard=cat)
    p.parse(itemName, useTMDb=(ARGS.tmdb_api_key is not None), hasIMDbId=imdbidstr, hasTMDbId=tmdbidstr)
    p.title = fixNtName(p.title)
    
    cat = genCatFolderName(p)

    destFolderName = genMediaFolderName(p)
    destCatFolderName = os.path.join(cat, destFolderName)

    if os.path.isfile(mediaSrc):
        filename, file_ext = os.path.splitext(itemName)
        if isMediaFileType(file_ext):
            if cat == CATNAME_TV:
                print('\033[33mSingle Episode file?  %s \033[0m' % mediaSrc)
                if ARGS.origin_name:
                    newTVFileName = itemName
                else:
                    newTVFileName = genTVSeasonEpisonGroup(
                        itemName, p.group, p.resolution)
                seasonFolderFullPath = os.path.join(ARGS.tv_folder_name, destFolderName,
                                                    p.season)
                targetCopy(mediaSrc, seasonFolderFullPath, newTVFileName)
                mkPlexMatch(os.path.join(ARGS.tv_folder_name, destFolderName), p)
                targetDirHook(os.path.join(ARGS.tv_folder_name, destFolderName), tmdbidstr=str(p.tmdbid))
            elif cat == CATNAME_MOVIE:
                if ARGS.origin_name:
                    newMovieName = itemName
                elif ARGS.tmdb_origin_name:
                    yearstr = str(p.year) if p.year > 0 else ''
                    newMovieName = genMovieTMDbOriginName(mediaSrc, p.title,
                                                    yearstr, nameParser=p)
                else:
                    yearstr = str(p.year) if p.year > 0 else ''
                    newMovieName = genMovieResGroup(mediaSrc, p.title,
                                                    yearstr, p.resolution,
                                                    p.group, nameParser=p)
                targetCopy(mediaSrc, destCatFolderName, newMovieName)
                targetDirHook(destCatFolderName, tmdbidstr=str(p.tmdbid))
            elif cat == 'TMDbNotFound':
                targetCopy(mediaSrc, cat)
                targetDirHook(os.path.join(cat, itemName), tmdbidstr=str(p.tmdbid))
            else:
                print('\033[33mSingle media file : [ %s ] %s \033[0m' %
                      (cat, mediaSrc))
                targetCopy(mediaSrc, destCatFolderName)
                targetDirHook(destCatFolderName, tmdbidstr=str(p.tmdbid))
        elif file_ext.lower() in ['.iso']:
            #  TODO: aruba need iso when extract_bdmv
            if ARGS.full_bdmv or ARGS.extract_bdmv:
                bdmvFolder = os.path.join('BDMVISO', destFolderName)
                targetCopy(mediaSrc, bdmvFolder)
                targetDirHook(bdmvFolder, tmdbidstr='iso')
            else:
                print('\033[33mSkip .iso file:  %s \033[0m' % mediaSrc)
        else:
            print('\033[34mSkip file:  %s \033[0m' % mediaSrc)
    else:
        if cat == CATNAME_TV:
            copyTVFolderItems(mediaSrc, destFolderName, p.season, p.group,
                              p.resolution, p)
        elif cat == CATNAME_MOVIE:
            processMovieDir(mediaSrc, p.ccfcat,
                            destFolderName, folderTmdbParser=p)
        elif cat in ['MV']:
            targetCopy(mediaSrc, cat)
            targetDirHook(os.path.join(cat, itemName), tmdbidstr='mv')
        elif cat in ['Music']:
            processMusic(mediaSrc, cat, destFolderName)
        elif cat in ['TMDbNotFound']:
            if p.tmdbcat == 'movie':
                print('\033[33mSearch media in dir: [ %s ], %s\033[0m ' %
                    (cat, mediaSrc))
                processMovieDir(mediaSrc, cat, destFolderName, folderTmdbParser=p)
            else:
                targetCopy(mediaSrc, cat)
                targetDirHook(os.path.join(cat, itemName), tmdbidstr='notfound')

        elif cat in ['HDTV', 'Audio']:
            targetCopy(mediaSrc, cat)
            targetDirHook(os.path.join(cat, itemName), tmdbidstr='audio')
        elif cat in ['eBook']:
            print('\033[33mSkip eBoook: [%s], %s\033[0m ' %
                  (cat, mediaSrc))
            # if you don't want to skip these, comment up and uncomment below
            # targetCopy(mediaSrc, p.cat)
        else:
            print('\033[33mDir treat as movie folder: [ %s ], %s\033[0m ' %
                  (cat, mediaSrc))
            processMovieDir(mediaSrc, cat, destFolderName, folderTmdbParser=p)


def makeKeepExts():
    global KEEPEXTS, KEEPEXTALL
    KEEPEXTALL = False
    KEEPEXTS = ['.mkv', '.mp4', '.ts', '.m2ts']
    if ARGS.keep_ext == 'all':
        KEEPEXTALL = True
        return
    if ARGS.keep_ext:
        argExts = ARGS.keep_ext.split(',')
        for ext in argExts:
            ext = ext.strip()
            if ext:
                if ext[0] == '.':
                    KEEPEXTS.append(ext)
                else:
                    KEEPEXTS.append('.' + ext)


def ensureIMDb():
    if ARGS.imdbid:
        m1 = re.search(r'(tt\d+)', ARGS.imdbid, re.A)
        if m1:
            ARGS.imdbid = m1[1]
        else:
            ARGS.imdbid = ''


def loadArgs():
    parser = argparse.ArgumentParser(
        description='torcp: a script hardlink media files and directories in Emby-happy naming and structs.'
    )
    parser.add_argument(
        'MEDIA_DIR',
        help='The directory contains TVs and Movies to be copied.')
    parser.add_argument('-d',
                        '--hd_path',
                        required=True,
                        help='the dest path to create Hard Link.')
    parser.add_argument('-e',
                        '--keep-ext',
                        help='keep files with these extention(\'srt,ass\').')
    parser.add_argument('-l',
                        '--lang',
                        help='seperate move by language(\'cn,en\').')
    parser.add_argument(
        '--tmdb-api-key',
        help='Search API for the tmdb id, and gen dirname as Name (year)\{tmdbid=xxx\}'
    )
    parser.add_argument('--tmdb-lang',
                        default='zh-CN',
                        help='specify the TMDb language')
    parser.add_argument('--tv-folder-name',
                        default='TV',
                        help='specify the name of TV directory, default TV.')
    parser.add_argument('--movie-folder-name',
                        default='Movie',
                        help='specify the name of Movie directory, default Movie.')
    parser.add_argument('--tv',
                        action='store_true',
                        help='specify the src directory is TV.')
    parser.add_argument('--movie',
                        action='store_true',
                        help='specify the src directory is Movie.')
    parser.add_argument('--dryrun',
                        action='store_true',
                        help='print message instead of real copy.')
    parser.add_argument('--single',
                        '-s',
                        action='store_true',
                        help='parse and copy one single folder.')
    parser.add_argument('--extract-bdmv',
                        action='store_true',
                        help='extract largest file in BDMV dir.')
    parser.add_argument('--full-bdmv',
                        action='store_true',
                        help='copy full BDMV dir and iso files.')
    parser.add_argument('--origin-name',
                        action='store_true',
                        help='keep origin file name.')
    parser.add_argument('--tmdb-origin-name',
                        action='store_true',
                        help='filename emby bracket - origin file name.')
    parser.add_argument('--sleep',
                        type=int,
                        help='sleep x seconds after operation.')
    parser.add_argument('--move-run',
                        action='store_true',
                        help='WARN: REAL MOVE...with NO REGRET.')
    parser.add_argument('--make-log',
                        action='store_true',
                        help='Make a log file.')
    parser.add_argument('--symbolink',
                        action='store_true',
                        help='symbolink instead of hard link')
    parser.add_argument('--cache',
                        action='store_true',
                        help='cache searched dir entries')
    parser.add_argument('--emby-bracket',
                        action='store_true',
                        help='ex: Alone (2020) [tmdbid=509635]')
    parser.add_argument('--filename-emby-bracket',
                        action='store_true',
                        help='filename with emby bracket')
    parser.add_argument('--plex-bracket',
                        action='store_true',
                        help='ex: Alone (2020) {tmdb-509635}')
    parser.add_argument('--make-plex-match',
                        action='store_true',
                        help='Create a .plexmatch file at the top level of a series')
    parser.add_argument('--after-copy-script',
                        default='',
                        help='call this script with destination folder path after link/move')
    parser.add_argument('--imdbid',
                        default='',
                        help='specify the IMDb id, -s single mode only')
    parser.add_argument('--tmdbid',
                        default='',
                        help='specify the TMDb id, -s single mode only')
    parser.add_argument('--site-str',
                        help='site-id(ex. hds-12345) folder name, set site strs like (\'chd,hds,ade,ttg\').')

    global ARGS
    ARGS = parser.parse_args()
    ensureIMDb()
    ARGS.MEDIA_DIR = os.path.expanduser(ARGS.MEDIA_DIR)
    makeKeepExts()


def hasIMDbId(str):
    m1 = re.search(r'\[imdb(id)?\=(tt\d+)\]', str.strip(), flags=re.A | re.I)
    m2 = re.search(r'(tt\d+)\s*$', str, flags=re.A | re.I)
    if m1:
        return m1[2]
    elif m2:
        return m2[1]
    else:
        return None


def matchSiteId(str):
    if ARGS.site_str:
        # argSites = ARGS.site_str.split(',')
        # siteStr = '|'.joint(argSites)
        siteStr = ARGS.site_str.replace(',', '|')
    else:
        siteStr = 'chd|hds|ourbits|hdc|ttg|ade|cmct|frds|pter|u2|mteam|hdh|lemon'
    # m1 = re.search(r'(chd|hds|ourbits|hdc|ttg|ade|cmct|frds|pter|u2|mteam|hdh|lemon)-(\d+)$', str.strip(), flags=re.A | re.I)
    m1 = re.search(r'('+siteStr+')-(\d+)$', str.strip(), flags=re.A | re.I)
    if m1:
        return m1[2]
    else:
        return None


def onlyOneDirInSiteIdFolder(cpLocation, foldername):
    siteid = matchSiteId(foldername)
    if siteid:
        # dirlist = [name for name in os.listdir(os.path.join(cpLocation, foldername)) if os.path.isdir(os.path.join(cpLocation, foldername, name))]
        dirlist = os.listdir(os.path.join(cpLocation, foldername))
        if len(dirlist) >= 1:
            return siteid, dirlist[0]
    return '', ''


def onlyOneDirInIMDbFolder(cpLocation, foldername):
    imdbstr = hasIMDbId(foldername)
    if imdbstr:
        dirlist = os.listdir(os.path.join(cpLocation, foldername))
        if len(dirlist) >= 1:
            return imdbstr, dirlist[0]
        # dirlist = [name for name in os.listdir(os.path.join(cpLocation, foldername)) if os.path.isdir(os.path.join(cpLocation, foldername, name))]
        # if len(dirlist) == 1:
            # return imdbstr, dirlist[0]
    return '', ''


def parseFolderIMDbId(locIn, itemIn):
    siteid, insideSiteFolderName = onlyOneDirInSiteIdFolder(locIn, itemIn)
    folderIMDb, insideIMDbFolderName = onlyOneDirInIMDbFolder(locIn, itemIn)
    if folderIMDb:
        parentLocation = os.path.join(locIn, itemIn)
        itemName = insideIMDbFolderName
    elif siteid:
        parentLocation = os.path.join(locIn, itemIn)
        itemName = insideSiteFolderName
    else:
        parentLocation = locIn
        itemName = itemIn
    return parentLocation, itemName, folderIMDb


def main():
    loadArgs()
    cpLocation = ARGS.MEDIA_DIR
    cpLocation = os.path.abspath(cpLocation)

    print("=========>>> " +
          datetime.datetime.now().astimezone().strftime("%Y-%m-%dT%H:%M:%S %z"))

    if ARGS.cache:
        searchCache = CacheManager(cpLocation)
        searchCache.openCache()

    argIMDb = ARGS.imdbid if (ARGS.single and ARGS.imdbid) else ''
    argTMDb = ARGS.tmdbid if (ARGS.single and ARGS.tmdbid) else ''

    if os.path.isfile(cpLocation):
        processOneDirItem(os.path.dirname(cpLocation),
                          os.path.basename(os.path.normpath(cpLocation)), imdbidstr=argIMDb, tmdbidstr=argTMDb)
    else:
        if ARGS.single and not isCollections(cpLocation):
            # processOneDirItem(os.path.dirname(cpLocation),
            #                   os.path.basename(os.path.normpath(cpLocation)), imdbidstr)
            
            parentLocation, itemName, folderimdb = parseFolderIMDbId(os.path.dirname(cpLocation),
                              os.path.basename(os.path.normpath(cpLocation)))
            if argIMDb or argTMDb:
                processOneDirItem(parentLocation, itemName, imdbidstr=argIMDb, tmdbidstr=argTMDb)
            else:
                processOneDirItem(parentLocation, itemName, folderimdb)

        else:
            for torFolderItem in os.listdir(cpLocation):
                if uselessFile(torFolderItem):
                    continue
                
                parentLocation, itemName, folderimdb = parseFolderIMDbId(cpLocation, torFolderItem)

                if isCollections(itemName) and os.path.isdir(
                        os.path.join(parentLocation, itemName)):
                    print('\033[35mProcess collections: %s \033[0m' %
                          itemName)
                    packDir = os.path.join(parentLocation, itemName)
                    for fn in os.listdir(packDir):
                        if ARGS.cache:
                            if searchCache.isCached(fn):
                                print('\033[32mSkipping. File previously linked: %s \033[0m' % (fn))
                                continue
                            else:
                                searchCache.append(fn)
                        processOneDirItem(packDir, fn, imdbidstr='')
                else:
                    if ARGS.cache:
                        if searchCache.isCached(itemName):
                            print('\033[32mSkipping. File previously linked: %s \033[0m' % (itemName))
                            continue
                        else:
                            searchCache.append(itemName)
                    processOneDirItem(parentLocation, itemName, folderimdb)

    if ARGS.cache:
        searchCache.closeCache()


if __name__ == '__main__':
    # logging.basicConfig(level=logging.DEBUG)
    main()



import os
import time
import shutil
import argparse


def dropEmptyFolders(directory):
    """Verify that every empty folder removed in local storage."""
    for diritem in os.listdir(directory):
        isEmpty = True
        dirpath = os.path.join(directory, diritem)
        for dir, dirs, files in os.walk(dirpath):
            if not dirs and not files:
                print('\033[33mDelete empty folder: %s\033[0m ' % dir)
                if not ARGS.dryrun:
                    if ARGS.sleep:
                        time.sleep(ARGS.sleep)
                    os.rmdir(dir)
            for fn in files:
                fullfilepath = os.path.join(dir, fn)
                filename, fileext = os.path.splitext(fn)
                if fileext.lower() in KEEPEXTS:
                    print('\033[32mNot Empty: %s\033[0m ' % fullfilepath)
                    isEmpty = False
                else:
                    if os.path.isfile(fullfilepath):
                        print('\033[33mDelete file: %s\033[0m ' % fullfilepath)
                        if not ARGS.dryrun:
                            if ARGS.sleep:
                                time.sleep(ARGS.sleep)
                            os.remove(fullfilepath)
        if isEmpty and os.path.isdir(dirpath):
            print('\033[31mrmtree: %s\033[0m ' % dirpath)
            if not ARGS.dryrun:
                if ARGS.sleep:
                    time.sleep(ARGS.sleep)
                shutil.rmtree(dirpath)
                # passs


def dropEmptyFolers2(directory):
    folders = list(os.walk(directory))[1:]

    for folder in folders:
        # folder example: ('FOLDER/3', [], ['file'])
        if not folder[2]:
            os.rmdir(folder[0])

def makeKeepExts():
    global KEEPEXTS
    KEEPEXTS = ['.mkv', '.mp4', '.ts', '.iso']
    if ARGS.keep_ext:
        argExts = ARGS.keep_ext.split(',')
        for ext in argExts:
            ext = ext.strip()
            if ext:
                if ext[0] == '.':
                    KEEPEXTS.append(ext)
                else:
                    KEEPEXTS.append('.' + ext)

def loadArgs():
    parser = argparse.ArgumentParser(
        description=
        'DeleteEmptyFolders: The script scan folders in FOLDER and delete if the sub-folders are empty.'
    )
    parser.add_argument('FOLDER', help='The directory to be scaned.')
    parser.add_argument('-e',
                        '--keep-ext',
                        help='folders will be delete if files lefted with extention not in (\'srt,ass\').')
    parser.add_argument('--sleep',
                        type=int,
                        help='sleep x seconds after operation.')
    parser.add_argument('--dryrun',
                        action='store_true',
                        help='print message instead of real delete.')

    global ARGS
    ARGS = parser.parse_args()
    ARGS.FOLDER = os.path.expanduser(ARGS.FOLDER)
    makeKeepExts()

def main():
    loadArgs()
    dropEmptyFolders(ARGS.FOLDER)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import re
import os


def cutExt(torName):
    if not torName:
        return ''
    tortup = os.path.splitext(torName)
    torext = tortup[1].lower()
    if re.match(r'\.[0-9a-z]{2,8}$', torext, flags=re.I):
    # mvext = ['.mkv', '.ts', '.m2ts', '.vob', '.mpg', '.mp4', '.3gp', '.mov', '.tp', '.zip', '.pdf', '.iso', '.ass', '.srt', '.7z', '.rar']
    # if torext.lower() in mvext:
        return tortup[0].strip()
    else:
        return torName


class CategoryItem:
    def __init__(label, number):
        label = label
        number = number
        # size = 0


class TorCategory:
    # def __init__(self):
    # 有些组生产 TV Series，但是在种子名上不显示 S01 这些
    TV_GROUPS = ['CMCTV', 'FLTTH']
    WEB_GROUPS = ['CHDWEB', 'PTerWEB', 'HaresWEB', 'DBTV', 'QHStudio', 
                 'LeagueWEB', 'HDCTV', '52KHD', 'PTHweb', 'OURTV', 'iLoveTV']

    # 有些组专门生产 MV
    MV_GROUPS = ['PTerMV', 'FHDMv', 'Melon', 'HaresMV', 'Bugs!']
    # 有些组专门生产 Audio
    AUDIO_GROUPS = ['PTHAudio', 'HDSAB']
    # 有些组专门作压制，但是不在种子名上标记
    MOVIE_ENCODE_GROUPS = ['CMCT', 'FRDS']

    CATEGORIES = {
        'TV': ['TV', '32', 0, 'TV'],
        'MV': ['MV', '31;1', 0, 'MV'],
        'Audio': ['Audio', '32;1', 0, 'Audio'],
        'Music': ['Music', '31', 0, 'Music'],
        'eBook': ['eBook', '34', 0, 'eBook'],
        # 压制 1080p and lower, 适合emby
        'MovieEncode': ['MovieEncode', '36', 0, 'MovieEncode'],
        # Remux 1080p and lower, 适合emby
        'MovieRemux': ['MovieRemux', '36', 0, 'MovieRemux'],
        'Movie4K': ['Movie4K', '36', 0, 'Movie4K'],  # 压制和Remux 4K，适合emby
        'MovieWebdl': ['MovieWebdl', '36', 0, 'MovieWebdl'],  # Web DL，适合emby
        'MovieWeb4K': ['MovieWeb4K', '36', 0, 'MovieWeb4K'],  # Web DL，适合emby
        # 原盘, 适合播放机 & kodi
        'MovieBDMV': ['MovieBDMV', '35', 0, 'MovieBDMV'],  # 原盘, 适合播放机 & kodi
        'MovieBDMV4K': ['MovieBDMV4K', '35', 0, 'MovieBDMV4K'],
        'MovieDVD': ['MovieDVD', '35', 0, 'MovieDVD'],
        'Movie': ['Movie', '35', 0, 'Movie'],
        'HDTV': ['HDTV', '33', 0, 'HDTV'],
        'Other': ['Other', '33', 0, 'Others'],
        'Archive': ['Archive', '33', 0, 'Archive']
    }

    category = ''
    group = ''
    resolution = ''
    quality = ''
    CategorySummary = []

    def __init__(self, torName):
        self.ccfcat, self.group = self.guessByName(torName)


    def setCategory(self, category):
        self.category = category
        self.CATEGORIES[category][2] += 1

    def categoryByExt(self, torName):
        if re.search(
                r'(pdf|epub|mobi|txt|chm|azw3|CatEDU|eBook-\w{4,8}|mobi|doc|docx).?$',
                torName, re.I):
            self.setCategory('eBook')
        elif re.search(
                r'(zip|7z|rar).?$',
                torName, re.I):
            self.setCategory('Archive')
        elif re.search(r'\.(mpg)\b', torName, re.I):
            self.setCategory('MV')
        elif re.search(r'(\b|_)(FLAC.{0,3}|DSF.{0,3}|DSD(\d{1,3})?)$', torName, re.I):
            self.setCategory('Music')
        elif re.search(r'(\b|_)(BD25\b|BD50\b|BD66\b|BD$)', torName, re.I):
            self.setCategory('MovieBDMV')
        elif re.search(r'(\b|_)(DVDR|DVD(\d+)?)\b', torName, re.I):
            self.setCategory('MovieDVD')
        else:
            return False
        return True

    def categoryByKeyword(self, torName):
        if re.search(r'(上下册|全.{1,4}册|精装版|修订版|第\d版|共\d本|文集|新修版|PDF版|课本|课件|出版社)',
                     torName):
            self.setCategory('eBook')
        elif re.search(r'(\d+册|\d+期|\d+版|\d+本|\d+年|\d+月|系列|全集|作品集).?$',
                       torName):
            self.setCategory('eBook')
        elif re.search(r'(\bConcert|演唱会|音乐会|\bLive[. ](At|in))\b', torName, re.A | re.I):
            self.setCategory('MV')
        elif re.search(r'\bBugs!.?\.mp4', torName, re.I):
            self.setCategory('MV')
        elif re.search(r'(\bVarious Artists|\bMQA\b|整轨|\b分轨|\b分軌|\b无损|\bLPCD|\bSACD|\bMP3|XRCD\d{1,3})',
                       torName, re.A | re.I):
            self.setCategory('Music')
        elif re.search(r'(\b\d+ ?CD|(\[|\()\s*(16|24)\b|\-(44\.1|88.2|48|192)|24Bit|44\s*\]|FLAC.*(16|24|48|CUE|WEB|Album)|WAV.*CUE|CD.*FLAC|(\[|\()\s*FLAC)', torName, re.A | re.I):
        # elif re.search(r'(\b\d+ ?CD|24\-|\-44\.1|24Bit|\[[\d\s]*44\s*\]|FLAC.*44|FLAC.*48|WAV.*CUE|FLAC.*CUE|\[.*FLAC\]|FLAC.+WEB\b|FLAC.*Album|CD[\s-]+FLAC|FLAC[\s-]+CD)', torName, re.A | re.I):
            self.setCategory('Music')
        elif re.search(r'^(Beethoven|Schubert)\s*[\-_]', torName, re.I):
            self.setCategory('Music')
        # elif re.search(r'(乐团|交响曲|协奏曲|奏鸣曲|[二三四]重奏|专辑\b)', torName):
        #     self.setCategory('Music')
        elif re.search(r'(\[BDMV\])', torName, re.I):
            self.setCategory('MovieBDMV')
        elif re.search(r'(\bThe.Movie.\d{4}|电影版)\b', torName, flags=re.A | re.I):
            if self.quality == 'WEBDL':
                self.setCategory('MovieWebdl')
            else:
                self.setCategory('MovieEncode')
        else:
            return False
        return True

    def categoryTvByName(self, torName):
        if re.search(r'(\b(S\d+)(E\d+)?|(Ep?\d+-Ep?\d+))\b', torName, flags=re.A | re.I):
            self.setCategory('TV')
        elif re.search(r'\b(Season\s?\d+)\b', torName, flags=re.A | re.I):
            self.setCategory('TV')
        elif re.search(r'(第\s*(\d+)(-\d+)?季)\b', torName, flags=re.I):
            self.setCategory('TV')
        elif re.search(r'(\bS\d+(-S\d+))\b', torName, flags=re.A | re.I):
            self.setCategory('TV')
        elif re.search(r'\W[ES]\d+\W|EP\d+\W|\d+季|第\w{1,3}季\W', torName, re.A | re.I):
            self.setCategory('TV')
        elif re.search(r'\bHDTV\b', torName):
            self.setCategory('HDTV')
        elif re.search(r'(Complete.+Web-?dl|Full.Season|The[\s\.]*(Complete\w*|Drama\w*|Animate\w*)?[\s\.]*Series|\d+集)', torName, re.A | re.I):
            self.setCategory('TV')
        else:
            return False
        return True

    def categoryMVAudioGroup(self, torName, group):
        if group in self.MV_GROUPS:
            self.setCategory('MV')
        elif group in self.AUDIO_GROUPS:
            self.setCategory('Audio')
        else:
            return False
        return True

    def categoryByGuessGroup(self, torName, group):
        filename, file_ext = os.path.splitext(torName)
        isMediaFile = file_ext.lower() in ['.mkv', '.mp4', '.iso']
            
        if group in self.TV_GROUPS and not isMediaFile:
            self.setCategory('TV')
        elif group in self.WEB_GROUPS and not isMediaFile:
            self.setCategory('TV')
        elif group in self.MOVIE_ENCODE_GROUPS:
            self.setCategory('MovieEncode')
        else:
            return False
        return True

    def parseGroup(self, torName):
        sstr = cutExt(torName)
        match = re.search(r'[@\-￡]\s?(\w+)(?!.*[@\-￡].*)$', sstr, re.I)
        if match:
            groupName = match.group(1).strip()
            # # TODO: BD-50_A_PORTRAIT_OF_SHUNKIN_1976_BC
            if match.span(1)[0] < 4:
                return None
            if groupName.startswith('CMCT') and not groupName.startswith('CMCTV'):
                groupName = 'CMCT'
            return groupName

        return None

    def getResolution(self, torName):
        match = re.search(r'\b(4K|2160p|1080[pi]|720p|576p|480p)\b', torName, re.A | re.I)
        if match:
            return match.group(0).strip().lower()
        else:
            return ''

    def getSource(self, torName):
        match = re.search(r'\b(Blu[\-\. ]?Ray|WEB|WEB[\-\. ]?DL|WEBRip|^BD([-. ]\d)*|BD$)\b', torName, re.A | re.I)
        if match:
            # mediaSource = match.group(0).strip().lower()
            if re.search(r'(\bBlu|\bBD)', match.group(0), flags=re.A | re.I):
                return 'BLURAY'
            else:
                return 'WEBDL'
        else:
            return ''

    def categoryByQuality(self, torName):
        # 来源为原盘的
        if self.quality == 'BLURAY':
            # Remux, 压制 还是 原盘
            if re.search(r'\WREMUX\W', torName, re.I):
                # if self.resolution == '2160p':
                #     self.setCategory('Movie4K')
                # else:
                #     self.setCategory('MovieRemux')
                self.setCategory('MovieRemux')
            elif re.search(r'\b(x265|x264)\b', torName, re.I):
                # if self.resolution == '2160p':
                #     self.setCategory('Movie4K')
                # else:
                #     self.setCategory('MovieEncode')
                self.setCategory('MovieEncode')
            else:
                if self.resolution == '2160p':
                    self.setCategory('MovieBDMV4K')
                else:
                    self.setCategory('MovieBDMV')
        # 来源是 WEB-DL
        elif self.quality == 'WEBDL': 
            # if self.resolution == '2160p':
            #     self.setCategory('MovieWeb4K')
            # else:
            #     self.setCategory('MovieWebdl')
            self.setCategory('MovieWebdl')
        else:
            if re.search(r'\WREMUX\W', torName, re.I):
                self.setCategory('MovieRemux')
            elif re.search(r'\b(x265|x264)\b', torName, re.I):
                self.setCategory('MovieEncode')
            else:
                return False
        return True

    def guessByName(self, torName):
        self.group = self.parseGroup(torName)
        self.quality = self.getSource(torName)
        self.resolution = self.getResolution(torName)

        if self.categoryMVAudioGroup(torName, self.group):
            return self.category, self.group

        if self.categoryByExt(torName):
            return self.category, self.group

        if self.categoryTvByName(torName):
            return self.category, self.group

        if self.categoryByKeyword(torName):
            return self.category, self.group

        # if self.categoryByGuessGroup(torName, self.group):
        #     return self.category, self.group

        # 非web组出的
        if self.categoryByQuality(torName):
            return self.category, self.group
        else:
            if self.resolution or self.quality:
                self.setCategory('Movie')
            else:
                # Other的条件： TV/MV/Audio都匹配不上，quality没标记，各种压制组也对不上
                self.setCategory('Other')
            return self.category, self.group

    def getSummary(self):
        for cat in self.CATEGORIES.keys():
            ic = CategoryItem(self.CATEGORIES[cat][0],
                              self.CATEGORIES[cat][2])
            self.CategorySummary.append(ic)
        return self.CategorySummary
